import ezdxf
from ezdxf import colors
import random
import math
from ezdxf.enums import TextEntityAlignment
import logging
from math import radians, cos, sin
from fractions import Fraction
import re
from ezdxf.math import Vec2
from ezdxf.render import mleader
from ezdxf.tools.text import MTextEditor

from createdrawing.section_view import draw_section_view
from createdrawing.front_elevation import draw_front_elevation
from createdrawing.common import *

from createdrawing.plenum_schedule import draw_plenum_schedule_table
from createdrawing.fan_schedule import draw_fan_schedule_table
from createdrawing.hood_schedule import draw_hood_schedule_table

logger = logging.getLogger('myapp')


# HOOD ACCESSORIES - RECTANGLES

def add_rectangle_with_circles(msp, bottom_left, width, height, circle_radius, layer):
    points = [
        bottom_left,
        (bottom_left[0] + width, bottom_left[1]),
        (bottom_left[0] + width, bottom_left[1] + height),
        (bottom_left[0], bottom_left[1] + height),
    ]
    msp.add_lwpolyline(points, close=True, dxfattribs={'layer': layer})

    circle_spacing = width / 4
    for i in range(1, 4):
        circle_center = (bottom_left[0] + i * circle_spacing, bottom_left[1] + height / 2)
        msp.add_circle(center=circle_center, radius=circle_radius, dxfattribs={'layer': layer})


# HOOD ACCESSORIES - TRIANGLE

def add_triangle(msp, start, end, vertex, dot_radius, layer):

    msp.add_line(start=start, end=end, dxfattribs={'layer': layer})
    msp.add_line(start=end, end=vertex, dxfattribs={'layer': layer})
    msp.add_line(start=vertex, end=start, dxfattribs={'layer': layer})

    centroid_x = (start[0] + end[0] + vertex[0]) / 3
    centroid_y = (start[1] + end[1] + vertex[1]) / 3
    centroid = (centroid_x, centroid_y)
    msp.add_circle(center=centroid, radius=dot_radius, dxfattribs={'layer': layer})


# DRAW LIGHTS

def draw_lights_with_circle(msp, bottom_left_x, bottom_left_y, width, height):
    bottom_left = (bottom_left_x, bottom_left_y)
    bottom_right = (bottom_left_x + width, bottom_left_y)
    top_right = (bottom_left_x + width, bottom_left_y + height)
    top_left = (bottom_left_x, bottom_left_y + height)
    
    points = [bottom_left, bottom_right, top_right, top_left]
    
    msp.add_lwpolyline(points, close=True, dxfattribs={'layer': 'HOOD ELECTRICAL'})
    
    radius = width / 2
    center_x = bottom_left_x + width / 2
    center_y = bottom_left_y + height / 2
    
    msp.add_circle((center_x, center_y), radius, dxfattribs={'layer': 'HOOD ELECTRICAL'})

    return bottom_right[0], bottom_right[1]


# DRAW SENSORS

def draw_sensor(msp, center_x, center_y, sensor_width, sensor_height, num_segments=10):
    sensor_radius = sensor_height / 2

    sensor_bottom_left_x = center_x - sensor_width / 2
    sensor_bottom_left_y = center_y - sensor_height / 2
    sensor_top_right_x = center_x + sensor_width / 2
    sensor_top_right_y = center_y + sensor_height / 2

    sensor_points = [
        (sensor_bottom_left_x, sensor_bottom_left_y),
        (sensor_top_right_x, sensor_bottom_left_y),
        (sensor_top_right_x, sensor_top_right_y),
        (sensor_bottom_left_x, sensor_top_right_y)
    ]

    msp.add_lwpolyline(sensor_points, close=True, dxfattribs={'layer': 'HOOD ELECTRICAL'})

    angle_step = 2 * math.pi / num_segments
    circle_points = []

    for i in range(num_segments):
        angle = i * angle_step
        x = center_x + sensor_radius * math.cos(angle)
        y = center_y + sensor_radius * math.sin(angle)
        circle_points.append((x, y))

    circle_points.append(circle_points[0])

    msp.add_lwpolyline(circle_points, close=True, dxfattribs={'layer': 'HOOD ELECTRICAL'})

    hatch = msp.add_hatch(color=colors.BLUE)
    hatch.paths.add_polyline_path(circle_points, is_closed=True)

    return (sensor_bottom_left_x, center_y), (sensor_top_right_x, center_y), sensor_top_right_y, sensor_width


# DRAW PLENUM COLLARS

def draw_mua_collars(msp, bottom_left_x, bottom_left_y, width, height, layer='HOOD'):
    bottom_left = (bottom_left_x, bottom_left_y)
    bottom_right = (bottom_left_x + width, bottom_left_y)
    top_right = (bottom_left_x + width, bottom_left_y + height)
    top_left = (bottom_left_x, bottom_left_y + height)
    
    points = [bottom_left, bottom_right, top_right, top_left]
    msp.add_lwpolyline(points, close=True, dxfattribs={'layer': layer})


def draw_vertical_dimension_line_plenum(msp, first_collar_x, x_offset=0, y_offset=0, height=30, dist=45, val="", layer='HOOD-ACCESSORIES', angle=90):
    dim = msp.add_linear_dim(
        base=(x_offset - dist, y_offset),  
        p1=(first_collar_x, y_offset),  
        p2=(x_offset, y_offset - height),  
        dimstyle=layer,
        override={
            "dimtsz": 0,  
            "dimasz": 1,  
            "dimdle": 2,
            "dimexe": 2,
            "dimltype": "DASHED2",
            "dimblk": "ARCHTICK",
            "dimtxt": 2,
            "dimclrd": 2,  # Dimension line color (2 corresponds to yellow in AutoCAD color index)
            "dimclre": 2,  # Extension line color
            "dimclrt": 2,  # Text color
        },
        text=val,
        angle=angle,  
    )
    
    dim.render()
    

def draw_vertical_dimension_line_plenum_right(msp, first_collar_x, x_offset=0, y_offset=0, height=30, dist=45, val="", layer='HOOD-ACCESSORIES', angle=90):
    dim = msp.add_linear_dim(
        base=(x_offset + dist, y_offset),  
        p1=(first_collar_x, y_offset),  
        p2=(x_offset, y_offset - height),  
        dimstyle=layer,
        override={
            "dimtsz": 0,  
            "dimasz": 1,  
            "dimdle": 2,
            "dimexe": 2,
            "dimltype": "DASHED2",
            "dimblk": "ARCHTICK",
            "dimtxt": 2,
            "dimclrd": 2,  # Dimension line color (2 corresponds to yellow in AutoCAD color index)
            "dimclre": 2,  # Extension line color
            "dimclrt": 2,  # Text color
        },
        text=val,
        angle=angle,  
    )
    
    dim.render()


def draw_horizontal_dimension_line_plenum(msp, hood_collar_bottom_y, x_offset=0, y_offset=0, height=30, width=111, val_center="", val_top="", layer='HOOD-ACCESSORIES'):
    
    dim = msp.add_linear_dim(
        base=(x_offset, y_offset - height - 10),
        p1=(x_offset, y_offset),
        p2=(x_offset + width, hood_collar_bottom_y),
        dimstyle=layer,
        override={
            "dimtsz": 0,
            "dimasz": 1,
            "dimdle": 2,
            "dimexe": 2,
            "dimblk": "ARCHTICK",
            "dimtxt": 2,
            "dimclrd": 2,
            "dimclre": 2,
            "dimclrt": 2,
        },
        angle=0,
        text=f"{val_top}\"",
    )
    
    dim.render()
    
    msp.add_text(
        text=val_center,
        dxfattribs={
            'insert': (x_offset + width / 2 - 3, y_offset - height - 10 - 5),  
            'layer': layer,
            'color': 2, 
            'height': 2
        }
    )


def get_text_width(text, height):
    return len(text) * height
    

def decimal_to_mixed_fraction(decimal_value):
    fraction = Fraction(decimal_value).limit_denominator(16)
    if fraction.denominator == 1:
        return f"{fraction.numerator}"
    else:
        whole_number = fraction.numerator // fraction.denominator
        remainder = fraction.numerator % fraction.denominator
        if whole_number == 0:
            return f"{remainder}/{fraction.denominator}"
        else:
            return f"{whole_number} {remainder}/{fraction.denominator}"


def parse_mixed_fraction(mixed_str):
    if ' ' in mixed_str:
        whole_number, fraction = mixed_str.split(' ')
        numerator, denominator = fraction.split('/')
        return int(whole_number) * 12 + Fraction(int(numerator), int(denominator))
    elif '/' in mixed_str:
        numerator, denominator = mixed_str.split('/')
        return Fraction(int(numerator), int(denominator))
    else:
        return int(mixed_str)


def inches_to_feet_and_inches(mixed_str):
    inches = parse_mixed_fraction(mixed_str)
    feet = int(inches // 1) // 12
    remainder_inches = inches - feet * 12
    if remainder_inches == 0:
        return f"{feet}'-0\""
    else:
        return f"{feet}'-{remainder_inches:.0f}\""


def draw_vertical_dimension_line(msp, x_offset=0, y_offset=0, height=30, width=111, val_center="", val_top="", layer='HOOD-ACCESSORIES', from_height = False, angle=0):
    
    custom_y_offset = (y_offset - height) if(from_height == False) else y_offset
    
    dim = msp.add_linear_dim(
        base=(x_offset, y_offset - height - 10),
        p1=(x_offset, custom_y_offset),
        # p1=(x_offset, y_offset),
        p2=(x_offset + width, custom_y_offset),
        # p2=(x_offset + width, y_offset),
        dimstyle=layer,
        override={
            "dimtsz": 0,
            "dimasz": 1,
            "dimdle": 2,
            "dimexe": 2,
            "dimblk": "ARCHTICK",
            "dimtxt": 2,
            "dimclrd": 2,  # Dimension line color (2 corresponds to yellow in AutoCAD color index)
            "dimclre": 2,  # Extension line color
            "dimclrt": 2,  # Text color
        },
        angle=angle,
        text=f"{val_top}\"",
    )
    
    dim.render()
    
    
    msp.add_text(
        text=val_center,
        dxfattribs={
            'insert': (x_offset + width / 2 - 3, y_offset - height - 10 - 5),  
            'layer': layer,
            'color': 2, 
            'height': 2
        }
    )


def draw_plan_view_info_label(msp, text='', start_x_offset=0, start_y_offset=0, pointer_x_offset=0, pointer_y_offset=0, alignment='left'):
    print(f"{text} => ${alignment}")
    ml_builder = msp.add_multileader_mtext("Standard")

    if alignment == 'right':
        text_alignment = mleader.TextAlignment.right
        connection_side = mleader.ConnectionSide.right
    elif alignment == 'left':
        text_alignment = mleader.TextAlignment.left
        connection_side = mleader.ConnectionSide.left
    elif alignment == 'bottom':
        text_alignment = mleader.TextAlignment.center
        connection_side = mleader.ConnectionSide.bottom
    else:
        text_alignment = mleader.TextAlignment.left
        connection_side = mleader.ConnectionSide.left
    
    rotation = 0
    
    ml_builder.set_content(
        text,
        alignment= text_alignment,
        color=colors.YELLOW,
        char_height=1.5,
    )

    # ml_builder.add_leader_line(mleader.ConnectionSide.right if alignment == 'right' else mleader.ConnectionSide.left, [Vec2(pointer_x_offset, pointer_y_offset)])
    ml_builder.add_leader_line(connection_side, [Vec2(pointer_x_offset, pointer_y_offset)])

    ml_builder.set_arrow_properties(size=2.0)
    ml_builder.set_leader_properties(
        color=colors.CYAN,
        lineweight=5,
    )
    
    # if alignment == 'bottom':
    #     rotation = 90
    # else:
    #     rotation = 0
    ml_builder.build(insert=Vec2(start_x_offset, start_y_offset), rotation=rotation)


# REMOTE CABINET INFO LABELS

def draw_remote_cabinet_info_label(msp, text='', start_x_offset=0, start_y_offset=0, pointer_x_offset=0, pointer_y_offset=0, alignment='bottom'):
    ml_builder = msp.add_multileader_mtext("Standard")

    if alignment == 'bottom':
        connection_side = mleader.ConnectionSide.bottom
        rotation = 270
        text_start_x_offset = start_x_offset + 0.5
        text_start_y_offset = start_y_offset - 3
    if alignment == 'right':
        connection_side = mleader.ConnectionSide.right
        rotation = 0
        text_start_x_offset = start_x_offset
        text_start_y_offset = start_y_offset

    ml_builder.add_leader_line(connection_side, [Vec2(pointer_x_offset, pointer_y_offset)])
    ml_builder.set_arrow_properties(size=2.0)
    ml_builder.set_leader_properties(
        color=colors.CYAN,
        lineweight=5,
    )
    ml_builder.build(insert=Vec2(start_x_offset, start_y_offset))
    mtext = msp.add_mtext(
        text,
        dxfattribs={
            'insert': (text_start_x_offset, text_start_y_offset),  
            'color': 2, 
            'rotation': rotation
        }
    )
    mtext.dxf.char_height = 1.5


# INFO LABEL FOR OUTER CYLINDER

def draw_plan_view_info_label_outer(msp, text='', start_x_offset=0, start_y_offset=0, pointer_x_offset=0, pointer_y_offset=0, alignment='left'):
    print(f"{text} => ${alignment}")
    ml_builder = msp.add_multileader_mtext("Standard")
    ml_builder.set_content(
        text,
        alignment= mleader.TextAlignment.right if alignment == 'right' else mleader.TextAlignment.left,
        color=colors.YELLOW,
        char_height=1.5,
    )
    ml_builder.add_leader_line(mleader.ConnectionSide.right if alignment == 'right' else mleader.ConnectionSide.left, [Vec2(pointer_x_offset, pointer_y_offset)])
    ml_builder.set_arrow_properties(size=2.0)
    ml_builder.set_leader_properties(
        color=colors.CYAN,
        lineweight=5,
    )
    
    ml_builder.build(insert=Vec2(start_x_offset, pointer_y_offset - 8))


# CONTROL PORTION - CABINET [PLAN VIEW]

def draw_plan_view_control_portion(cabinet_block, control_type, cab_width, cab_location, control_portion_type, cabinet_type, cab_bottom_left, cab_bottom_right, cab_top_right, cab_top_left, dxfattribs={'layer': 'HOOD ACCESSORIES'}, text="Control"):

    x_offset = 0
    y_offset = 0

    if cabinet_type == 'remotecab':
        rotation = 0
        control_bottom_left = x_offset, y_offset - cab_width
        control_bottom_right = x_offset + control_type, y_offset - cab_width
        control_top_right = x_offset + control_type, y_offset
        control_top_left = x_offset, y_offset
    elif cabinet_type == 'hoodcab':
        rotation = 90
        if(cab_location.lower() == 'left'):
            control_bottom_left = x_offset, y_offset
            control_bottom_right = x_offset + cab_width, y_offset
            control_top_right = x_offset + cab_width, y_offset + control_type
            control_top_left = x_offset, y_offset + control_type
        elif(cab_location.lower() == 'right'):
            control_bottom_left = x_offset, y_offset
            control_bottom_right = x_offset + cab_width, y_offset
            control_top_right = x_offset + cab_width, y_offset + control_type
            control_top_left = x_offset, y_offset + control_type

    cabinet_block.add_lwpolyline([control_bottom_left, control_bottom_right, control_top_right, control_top_left], close=True, dxfattribs=dxfattribs)

    if cabinet_type == 'remotecab':
        circle_radius = 0.5
        circle_start_x = x_offset + (control_type / 4)
        vertical_center = y_offset - 1.5
        circle_spacing = 2
        circle_positions = [
            (circle_start_x, vertical_center),
            (circle_start_x + circle_spacing, vertical_center),
            (circle_start_x + 2 * circle_spacing, vertical_center)
        ]
        for pos in circle_positions:
            cabinet_block.add_circle(center=pos, radius=circle_radius, dxfattribs=dxfattribs)

        text_height = 1.0
        total_height = (text_height * 2)
        box_center_x = x_offset + ( control_type / 2)
        box_center_y = y_offset - 2
        text_y_1 = box_center_y - (total_height / 2) - text_height
        text_y_2 = box_center_y - (total_height / 2) - text_height - 2
        text_x_1 = box_center_x - 1
        text_x_2 = box_center_x - 1

        switch_height = 1.5
        switch_width = 5
        switch_bottom_left_x = x_offset + (cab_width / 2) - (switch_width / 2)
        switch_bottom_left_y = y_offset  - cab_width

    elif cabinet_type == 'hoodcab':
        circle_radius = .5
        circle_start_x = x_offset + 1.5
        vertical_center = y_offset + (control_type / 2)
        circle_spacing = 2
        circle_positions = [
            (circle_start_x, vertical_center - circle_spacing),
            (circle_start_x, vertical_center),
            (circle_start_x, vertical_center + circle_spacing)
        ]
        for pos in circle_positions:
            cabinet_block.add_circle(center=pos, radius=circle_radius, dxfattribs=dxfattribs)

        text_height = 1.0
        total_height = (text_height * 2)
        box_center_x = x_offset + (cab_width / 2)
        box_center_y = y_offset + (control_type / 2)
        text_y_1 = box_center_y + (total_height / 2) - text_height
        text_y_2 = box_center_y - (total_height / 2) + text_height
        text_x_1 = box_center_x - 1
        text_x_2 = box_center_x + 1  

        switch_height = 1.5
        switch_width = 5
        switch_bottom_left_x = x_offset + (cab_width / 2) - (switch_width / 2)
        switch_bottom_left_y = y_offset  

    # Adding text inside the control portion box

    text_entity_1 = cabinet_block.add_text(
        'CONTROL',
        dxfattribs={
            'layer': 'TEXT',
            'height': text_height,
            'rotation': rotation
        }
    )
    text_entity_1.set_placement(
        (text_x_1, text_y_1),
        align=TextEntityAlignment.MIDDLE_CENTER
    )

    text_entity_2 = cabinet_block.add_text(
        'PANEL',
        dxfattribs={
            'layer': 'TEXT',
            'height': text_height,
            'rotation': rotation
        }
    )
    text_entity_2.set_placement(
        (text_x_2, text_y_2),
        align=TextEntityAlignment.MIDDLE_CENTER
    )

    # Drawing switch inside the control portion box
    if cabinet_type == 'hoodcab':
        cabinet_block.add_lwpolyline(
            points=[
                (switch_bottom_left_x, switch_bottom_left_y),
                (switch_bottom_left_x + switch_width, switch_bottom_left_y),
                (switch_bottom_left_x + switch_width, switch_bottom_left_y + switch_height),
                (switch_bottom_left_x, switch_bottom_left_y + switch_height),
                (switch_bottom_left_x, switch_bottom_left_y)
            ],
            dxfattribs={
                'layer': 'HOOD ELECTRICAL',
            }
        )

        switch_cont_height = 1
        switch_cont_width = 0.4
        switch_cont_distance = 1
        # Drawing 1st switch control
        cabinet_block.add_lwpolyline(
            points=[
                (switch_bottom_left_x + switch_width - switch_cont_distance, switch_bottom_left_y),
                (switch_bottom_left_x + switch_width - switch_cont_distance - switch_cont_width, switch_bottom_left_y),
                (switch_bottom_left_x + switch_width - switch_cont_distance - switch_cont_width, switch_bottom_left_y - switch_cont_height),
                (switch_bottom_left_x + switch_width - switch_cont_distance, switch_bottom_left_y - switch_cont_height),
                (switch_bottom_left_x + switch_width - switch_cont_distance, switch_bottom_left_y)
            ],
            dxfattribs={
                'layer': 'HOOD ELECTRICAL',
            }
        )
        # Drawing 2nd switch control
        cabinet_block.add_lwpolyline(
            points=[
                (switch_bottom_left_x + switch_cont_distance, switch_bottom_left_y),
                (switch_bottom_left_x + switch_cont_distance + switch_cont_width, switch_bottom_left_y),
                (switch_bottom_left_x + switch_cont_distance + switch_cont_width, switch_bottom_left_y - switch_cont_height),
                (switch_bottom_left_x + switch_cont_distance, switch_bottom_left_y - switch_cont_height),
                (switch_bottom_left_x + switch_cont_distance, switch_bottom_left_y)
            ],
            dxfattribs={
                'layer': 'HOOD ELECTRICAL',
            }
        )


def draw_plan_view_buckeye_cylinder(cabinet_block, x_offset, y_offset, outer_cylinder_qty, close=True, dxfattribs={'layer': 'HOOD ACCESSORIES'}, cab_location='right', cab_width=12, cylinder_radius = 3.8, outer_cylinder = False, cabinet_type='hoodcab'):

    center_x = x_offset
    center_y = y_offset
    
    outer_radius = cylinder_radius
    
    cabinet_block.add_circle((center_x, center_y), outer_radius, dxfattribs=dxfattribs)
    
    middle_radius = 1.5
    cabinet_block.add_circle((center_x, center_y), middle_radius, dxfattribs=dxfattribs)
    
    inner_radius = 0.5
    cabinet_block.add_circle((center_x, center_y), inner_radius, dxfattribs=dxfattribs)
    
    inner_radius2 = 0.35
    cabinet_block.add_circle((center_x, center_y), inner_radius2, dxfattribs=dxfattribs)
    
    small_center_radius = 0.08
    cabinet_block.add_circle((center_x, center_y), small_center_radius, dxfattribs=dxfattribs)
    
    if (cabinet_type == 'hoodcab' and outer_cylinder) or (cabinet_type == 'remotecab' and not outer_cylinder):
        eye_stand_points = [
            (center_x - 0.6, center_y - middle_radius),
            (center_x - 0.6, center_y - middle_radius - 0.05 - 0.25),
            (center_x + 0.6, center_y - middle_radius - 0.05 - 0.25),
            (center_x + 0.6, center_y - middle_radius),
        ]
        eye_points = [
            (center_x - 1, center_y - middle_radius - 0.05 - 0.25),
            (center_x + 1, center_y - middle_radius - 0.05 - 0.25),
            (center_x + 1, center_y - middle_radius - 0.05 - 0.25 - 0.1),
            (center_x + 1 - 0.15, center_y - middle_radius - 0.05 - 0.25 - 0.1),
            (center_x + 1 - 0.15, center_y - middle_radius - 0.05 - 0.25 - 0.4),
            (center_x - 1 + 0.15, center_y - middle_radius - 0.05 - 0.25 - 0.4),
            (center_x - 1 + 0.15, center_y - middle_radius - 0.05 - 0.25 - 0.1),
            (center_x - 1, center_y - middle_radius - 0.05 - 0.25 - 0.1),
            (center_x - 1, center_y - middle_radius - 0.05 - 0.25),
            (center_x - 1, center_y - middle_radius - 0.05 - 0.25 - 0.1),
            (center_x + 1, center_y - middle_radius - 0.05 - 0.25 - 0.1),
        ]
    else:
        if(cab_location.lower() == 'left') or (cabinet_type == 'remotecab' and outer_cylinder):
            eye_stand_points = [
                (center_x - middle_radius - 0.05, center_y + 0.6),
                (center_x - middle_radius - 0.05 - 0.25, center_y + 0.6),
                (center_x - middle_radius - 0.05 - 0.25, center_y - 0.6),
                (center_x - middle_radius - 0.05, center_y - 0.6),
            ]
            eye_points = [
                (center_x - middle_radius - 0.3, center_y - 1),
                
                (center_x - middle_radius - 0.3, center_y - 1),
                (center_x - middle_radius - 0.3, center_y - 1 + 2),
                (center_x - middle_radius - 0.3 - 0.09, center_y - 1 + 2),
                
                (center_x - middle_radius - 0.3 - 0.09, center_y - 1 + 2 - 0.1),
                (center_x - middle_radius - 0.3 - 0.09 - 0.3, center_y - 1 + 2 - 0.1),
                (center_x - middle_radius - 0.3 - 0.09 - 0.3, center_y - 1 + 2 - 0.1 - 1.8),
                (center_x - middle_radius - 0.3 - 0.09, center_y - 1 + 2 - 0.1 - 1.8),
                
                (center_x - middle_radius - 0.3 - 0.09, center_y - 1 + 2),
                (center_x - middle_radius - 0.3 - 0.09, center_y - 1),
                (center_x - middle_radius - 0.3, center_y - 1),
                (center_x - middle_radius - 0.3, center_y),
            ]
        elif(cab_location.lower() == 'right'):
            eye_stand_points = [
                (center_x + middle_radius - 0.05, center_y + 0.6),
                (center_x + middle_radius - 0.05 + 0.25, center_y + 0.6),
                (center_x + middle_radius - 0.05 + 0.25, center_y - 0.6),
                (center_x + middle_radius - 0.05, center_y - 0.6),
            ]
            eye_points = [
                (center_x + middle_radius + 0.2, center_y - 1),
                
                (center_x + middle_radius + 0.2, center_y - 1),
                (center_x + middle_radius + 0.2, center_y - 1 + 2),
                (center_x + middle_radius + 0.2 + 0.09, center_y - 1 + 2),
                
                (center_x + middle_radius + 0.2 + 0.09, center_y - 1 + 2 - 0.1),
                (center_x + middle_radius + 0.2 + 0.09 + 0.3, center_y - 1 + 2 - 0.1),
                (center_x + middle_radius + 0.2 + 0.09 + 0.3, center_y - 1 + 2 - 0.1 - 1.8),
                (center_x + middle_radius + 0.2 + 0.09, center_y - 1 + 2 - 0.1 - 1.8),
                
                (center_x + middle_radius + 0.2 + 0.09, center_y - 1 + 2),
                (center_x + middle_radius + 0.2 + 0.09, center_y - 1),
                (center_x + middle_radius + 0.2, center_y - 1),
                (center_x + middle_radius + 0.2, center_y),
            ]

    cabinet_block.add_lwpolyline(eye_stand_points, close=False, dxfattribs=dxfattribs)
    cabinet_block.add_lwpolyline(eye_points, close=False, dxfattribs=dxfattribs)

    # DRAW CYLINDER LABEL FOR REMOTE CABINET
    if cabinet_type == 'remotecab' and not outer_cylinder:
        draw_remote_cabinet_info_label(
            cabinet_block, 
            text=f'BUCKEYE\n CYLINDER',
            start_x_offset = center_x, start_y_offset = center_y - cylinder_radius - 10, 
            pointer_x_offset = center_x, pointer_y_offset = center_y - cylinder_radius,
            alignment='bottom'
        )
    elif cabinet_type == 'remotecab' and outer_cylinder:
        draw_remote_cabinet_info_label(
            cabinet_block, 
            text=f'BUCKEYE\n CYLINDER',
            start_x_offset = center_x + cylinder_radius + 8, start_y_offset = center_y, 
            pointer_x_offset = center_x + cylinder_radius, pointer_y_offset = center_y,
            alignment='right'
        )

    # DRAW CYLINDER LABEL FOR HOOD CABINET
    if cabinet_type == 'hoodcab':
        if outer_cylinder:
            if(cab_location.lower() == 'left'):
                draw_plan_view_info_label_outer(
                    cabinet_block,
                    start_x_offset=0 - 50,
                    start_y_offset=center_y + 0.5,
                    pointer_x_offset=center_x - 4,
                    pointer_y_offset=center_y - 0.25,
                    alignment='right',
                    text=f'({outer_cylinder_qty}) BUCKEYE\nCYLINDER'
                )
            else:
                draw_plan_view_info_label_outer(
                    cabinet_block,
                    start_x_offset=0 + 70,
                    start_y_offset=0,
                    pointer_x_offset=center_x + 4,
                    pointer_y_offset=center_y,
                    alignment='left',
                    text=f'({outer_cylinder_qty}) BUCKEYE\nCYLINDER'
                )
        else:
            if(cab_location.lower() == 'left'):
                draw_plan_view_info_label(
                    cabinet_block, 
                    start_x_offset=center_x - 20,
                    start_y_offset=center_y + 0.5,
                    pointer_x_offset=center_x - 4,
                    pointer_y_offset=center_y - 0.25,
                    alignment='right',
                    text='BUCKEYE\nCYLINDER',
                )
            else:
                draw_plan_view_info_label(
                    cabinet_block, 
                    start_x_offset=center_x + 20,
                    start_y_offset=center_y + 0.5,
                    pointer_x_offset=center_x + 4,
                    pointer_y_offset=center_y,
                    alignment='left',
                    text='BUCKEYE\nCYLINDER',
                )


def draw_plan_view_ansul_cylinder(cabinet_block, x_offset=0, y_offset=0, close=True, dxfattribs={'layer': 'HOOD ACCESSORIES'}, cab_location='right', cab_width=12, cylinder_radius = 3.8, outer_cylinder = False, outer_cylinder_qty=0, add_outer_label=False, cabinet_type=''):

    center_x = x_offset
    center_y = y_offset
    
    outer_radius = cylinder_radius
    
    cabinet_block.add_circle((center_x, center_y), outer_radius, dxfattribs=dxfattribs)
    
    middle_radius = 1
    cabinet_block.add_circle((center_x, center_y), middle_radius, dxfattribs=dxfattribs)
    
    inner_radius = 0.5
    cabinet_block.add_circle((center_x, center_y), inner_radius, dxfattribs=dxfattribs)
    
    hexagon_radius = (middle_radius + inner_radius) / 2
    hexagon_vertices = [
        (
            center_x + hexagon_radius * math.cos(math.radians(angle)),
            center_y + hexagon_radius * math.sin(math.radians(angle))
        )
        for angle in range(0, 360, 60)
    ]
    
    cabinet_block.add_lwpolyline(hexagon_vertices, close=True, dxfattribs=dxfattribs)
    
    inner_radius2 = 0.35
    cabinet_block.add_circle((center_x, center_y), inner_radius2, dxfattribs=dxfattribs)
    
    small_center_radius = 0.08
    cabinet_block.add_circle((center_x, center_y), small_center_radius, dxfattribs=dxfattribs)

    # DRAW CYLINDER LABEL FOR REMOTE CABINET
    if cabinet_type == 'remotecab' and not outer_cylinder:
        draw_remote_cabinet_info_label(
            cabinet_block, 
            text=f'ANSUL 3GAL\nCYLINDERS',
            start_x_offset = center_x, start_y_offset = center_y - cylinder_radius - 10, 
            pointer_x_offset = center_x, pointer_y_offset = center_y - cylinder_radius,
            alignment='bottom'
        )
    # elif cabinet_type == 'remotecab' and outer_cylinder:
    #     draw_plan_view_info_label(
    #         cabinet_block,
    #         start_x_offset=center_x + 25,
    #         start_y_offset=center_y + 0.6,
    #         pointer_x_offset=center_x + 10,
    #         pointer_y_offset=center_y,
    #         alignment='left',
    #         text=f'ANSUL REGULATED \nACTUATOR AND \nSINGLE TANK \nENCLOSURE WITH \n ({outer_cylinder_qty}) 3GAL \nCYLINDER.'
    #     )

    # DRAW CYLINDER LABEL FOR HOOD CABINET
    
    if cabinet_type == 'hoodcab' and not outer_cylinder:
        if(cab_location.lower() == 'left'):
            draw_plan_view_info_label(
                cabinet_block, 
                start_x_offset=center_x - 20,
                start_y_offset=center_y + 1,
                pointer_x_offset=center_x + 6 - cab_width,
                pointer_y_offset=center_y,
                alignment='right',
                text='ANSUL 3GAL\nCYLINDERS',
            )
        else:
            draw_plan_view_info_label(
                cabinet_block, 
                start_x_offset=center_x + 20,
                start_y_offset=center_y + 1,
                pointer_x_offset=center_x + 4,
                pointer_y_offset=center_y,
                alignment='left',
                text='ANSUL 3GAL\nCYLINDERS',
            )
    elif  cabinet_type == 'hoodcab' and  outer_cylinder:
        if(add_outer_label):
            if(cab_location.lower() == 'left'):
                draw_plan_view_info_label(
                    cabinet_block,
                    start_x_offset=center_x - 50,
                    start_y_offset=center_y + 0.6,
                    pointer_x_offset=center_x - 25,
                    pointer_y_offset=center_y,
                    alignment='right',
                    text=f'ANSUL REGULATED \nACTUATOR AND \nSINGLE TANK \nENCLOSURE WITH \n ({outer_cylinder_qty}) 3GAL \nCYLINDER.'
                )
            else:
                draw_plan_view_info_label(
                    cabinet_block,
                    start_x_offset=center_x + 50,
                    start_y_offset=center_y + 0.6,
                    pointer_x_offset=center_x + 25,
                    pointer_y_offset=center_y,
                    alignment='left',
                    text=f'ANSUL REGULATED \nACTUATOR AND \nSINGLE TANK \nENCLOSURE WITH \n ({outer_cylinder_qty}) 3GAL \nCYLINDER.'
                )


def draw_plan_view_fire_portion(cab_size, fire_portion_size, control_type, cabinet_block, cab_bottom_left, cab_bottom_right, cab_top_right, cab_top_left, close=True, dxfattribs={'layer': 'FP COMPONENTS'}, cabinet={}):
    
    cab_location = cabinet['cab_location']
    cab_width = cabinet['cab_width']
    tank_qty = cabinet['tank_qty'].lower()
    fire_portion_size = abs(fire_portion_size)

    cabinet_type = cabinet['cabinet_type']

    tank_qty = tank_qty.split('-')
    tank_type = tank_qty[0]
    tank_qty = int(tank_qty[1])
    
    # SRM-MECH-RELEASE

    srm_mech_release_width = 7
    srm_mech_release_height = 4.5
    x_offset_cab = 0
    y_offset_cab = 0
    
    if(cabinet_type == 'remotecab'):
        srm_bottom_left = control_type + 2, y_offset_cab - (cab_width/2 - srm_mech_release_width/2)
        srm_bottom_right = control_type + 2 + srm_mech_release_height, y_offset_cab - (cab_width/2 - srm_mech_release_width/2)
        srm_top_right = control_type + 2 +  + srm_mech_release_height, y_offset_cab - (cab_width/2 - srm_mech_release_width/2) - srm_mech_release_width
        srm_top_left = control_type + 2,  y_offset_cab - (cab_width/2 - srm_mech_release_width/2) - srm_mech_release_width
    elif(cabinet_type == 'hoodcab'):
        if(cab_location.lower() == 'left'):
            srm_bottom_left = (cab_width/2 - srm_mech_release_width/2), control_type + 1
            srm_bottom_right = (cab_width/2 + srm_mech_release_width/2), control_type + 1
            srm_top_right = (cab_width/2 + srm_mech_release_width/2), control_type + 1 + srm_mech_release_height
            srm_top_left = (cab_width/2 - srm_mech_release_width/2), control_type + 1 + srm_mech_release_height
        elif(cab_location.lower() == 'right'):
            srm_bottom_left = (cab_width/2 - srm_mech_release_width/2), control_type + 1
            srm_bottom_right = (cab_width/2 + srm_mech_release_width/2), control_type + 1
            srm_top_right = (cab_width/2 + srm_mech_release_width/2), control_type + 1 + srm_mech_release_height
            srm_top_left = (cab_width/2 - srm_mech_release_width/2), control_type + 1 + srm_mech_release_height

    fire_portion_controller_points = [srm_bottom_left, srm_bottom_right, srm_top_right, srm_top_left]
    
    srm_center_y =  srm_top_right[1] - (srm_mech_release_height / 2)

    pointer_x_offset = srm_top_right[0] + 1
    start_x_offset = pointer_x_offset + 15
    
    if(tank_type.lower() == 'ansul'):
        ansul_automan_width = cab_width / 2
        ansul_automan_height = 12

        if(cabinet_type == 'remotecab'):
            ansul_automan_bottom_left = control_type + 2, y_offset_cab
            ansul_automan_bottom_right = control_type + 2, y_offset_cab - ansul_automan_width
            ansul_automan_top_right = control_type + 2 + ansul_automan_height, y_offset_cab - ansul_automan_width
            ansul_automan_top_left = control_type + 2 + ansul_automan_height, y_offset_cab
        elif(cabinet_type == 'hoodcab'):
            if(cab_location.lower() == 'left'):
                ansul_automan_bottom_left = 0 + cab_width - ansul_automan_width, control_type + 1
                ansul_automan_bottom_right = 0 + cab_width, control_type + 1
                ansul_automan_top_right = 0 + cab_width, control_type + 1 + ansul_automan_height
                ansul_automan_top_left = 0 + cab_width - ansul_automan_width, control_type + 1 + ansul_automan_height
            elif(cab_location.lower() == 'right'):
                ansul_automan_bottom_left = 0, control_type + 1
                ansul_automan_bottom_right = ansul_automan_width, control_type + 1
                ansul_automan_top_right = ansul_automan_width, control_type + 1 + ansul_automan_height
                ansul_automan_top_left = 0, control_type + 1 + ansul_automan_height
        
        fire_portion_controller_points = [ansul_automan_bottom_left, ansul_automan_bottom_right, ansul_automan_top_right, ansul_automan_top_left]
        
        srm_center_y =  ansul_automan_top_right[1] - (ansul_automan_height / 2)

        pointer_x_offset = ansul_automan_top_right[0] + 1
        start_x_offset = pointer_x_offset + 15

    cabinet_block.add_lwpolyline(fire_portion_controller_points, close=True, dxfattribs=dxfattribs)

    fire_portion_text = 'SRM-MECH-RELEASE'
    
    if(tank_type.lower() == 'ansul'):
        fire_portion_text = 'ANSUL AUTOMAN'
    if(cab_location.lower() == 'left'):
        pointer_x_offset = srm_bottom_left[0] - 1
        start_x_offset = pointer_x_offset - 15
    elif(cab_location.lower() == 'right'):
        pointer_x_offset = srm_top_right[0] + 1
        start_x_offset = pointer_x_offset + 15

    if(cabinet_type == 'remotecab'):
        if(tank_type.lower() == 'ansul'):
            draw_remote_cabinet_info_label(
                cabinet_block, 
                text=fire_portion_text,
                start_x_offset = control_type + 2 + ansul_automan_height/2, start_y_offset = 0 - ansul_automan_width - 12, 
                pointer_x_offset = control_type + 2 + ansul_automan_height/2, pointer_y_offset = 0 - ansul_automan_width,
                alignment='bottom'
            )
        else:
            draw_remote_cabinet_info_label(
                cabinet_block, 
                text=fire_portion_text,
                start_x_offset = control_type + 2 + srm_mech_release_height/2, start_y_offset = 0 - cab_width - 8, 
                pointer_x_offset = control_type + 2 + srm_mech_release_height/2, pointer_y_offset = 0 - cab_width/2 - srm_mech_release_width/2,
                alignment='bottom'
            )
    elif(cabinet_type == 'hoodcab'):
        if(cab_location.lower() == 'left'):
            draw_plan_view_info_label(
                cabinet_block, 
                start_x_offset=start_x_offset - 10,
                start_y_offset=srm_center_y + 0.5,
                pointer_x_offset=start_x_offset + 10,
                pointer_y_offset=srm_center_y,
                alignment='right',
                text=fire_portion_text,
            )
        else:
            draw_plan_view_info_label(
                cabinet_block, 
                start_x_offset=start_x_offset,
                start_y_offset=srm_center_y + 0.5,
                pointer_x_offset=pointer_x_offset - 1,
                pointer_y_offset=srm_center_y,
                alignment='left',
                text=fire_portion_text,
            )

    # CYLINDERS

    cylinder_radius = 3.5
    cylinder_diameter = cylinder_radius * 2
    gap_between_cylinders = 2
    cylinder_height = cylinder_diameter + gap_between_cylinders

    if(tank_type.lower() == 'buckeye'):
        if cabinet_type == 'hoodcab':
            x_offset_inside = cab_width / 2
            x_offset_outside = cab_width + 15
            y_offset = control_type + srm_mech_release_height * 2.5
            if cab_location == 'right':
                current_y_position = y_offset
                current_x_offset_outside = x_offset_outside
            elif cab_location == 'left':
                current_y_position = y_offset
                current_x_offset_outside = 0 + cab_width - 15
        elif cabinet_type == 'remotecab':
            x_offset_inside = control_type + srm_mech_release_height * 2.5
            x_offset_outside = cab_size - 2 - cylinder_radius
            y_offset = 0 - cab_width / 2
            current_y_position = y_offset - 12
    
    if(tank_type.lower() == 'ansul'):
        if cabinet_type == 'hoodcab':
            x_offset_outside = cab_width + 15
            y_offset = control_type + ansul_automan_height + 5
            gap_between_cylinders = 1
            cylinder_height = cylinder_diameter + gap_between_cylinders
            current_y_position = y_offset
            if cab_location == 'right':
                x_offset_inside = 0 + cylinder_radius
                current_y_position = y_offset
                current_x_offset_outside = x_offset_outside
            elif cab_location == 'left':
                x_offset_inside = cab_width - cylinder_radius
                current_x_offset_outside = cab_width - 30
        elif cabinet_type == 'remotecab':
            x_offset_inside = control_type + 2 + ansul_automan_height + gap_between_cylinders + cylinder_radius
            x_offset_outside = cab_size - 2 - cylinder_radius
            y_offset = 0 - cylinder_radius
            current_y_position = 0 - cab_width - 2 - cylinder_radius

    outer_cylinder_qty  = 0
    outer_cylinder = False
    inner_cylinder_qty = 0
    actuator_count = 0
    add_outer_label = True
    outer_cylider_idx = 0

    for cylinder in range(tank_qty):

        if cabinet_type == 'remotecab':
            if x_offset_inside + cylinder_height > cab_size: # for outer cylinder
                x_offset = x_offset_outside
                current_y_position_outside = current_y_position
                current_y_position -= cylinder_diameter + gap_between_cylinders 
                outer_cylinder = True
                outer_cylinder_qty =  tank_qty - inner_cylinder_qty
            else: # for inside cylinders
                x_offset = x_offset_inside
                current_y_position_outside = y_offset
                inner_cylinder_qty += 1

        elif cabinet_type == 'hoodcab':
            if current_y_position + cylinder_height > cab_size:
                x_offset = current_x_offset_outside
                current_y_position_outside = cab_top_right[1] - cylinder_radius

                if(cab_location.lower() == 'left'):
                    current_x_offset_outside -= cylinder_diameter + gap_between_cylinders 
                elif(cab_location.lower() == 'right'):
                    current_x_offset_outside += cylinder_diameter + gap_between_cylinders 

                outer_cylinder = True
                outer_cylinder_qty =  tank_qty - inner_cylinder_qty
            else:
                x_offset = x_offset_inside
                current_y_position_outside = current_y_position
                inner_cylinder_qty += 1

        if(tank_type.lower() == 'buckeye'):
            draw_plan_view_buckeye_cylinder(
                cabinet_block, 
                x_offset, 
                current_y_position_outside,
                outer_cylinder_qty,
                close=close, 
                dxfattribs=dxfattribs, 
                cab_location=cab_location,
                cab_width=cab_width, 
                cylinder_radius = cylinder_radius,
                outer_cylinder = outer_cylinder,
                cabinet_type = cabinet_type,
            )
            
            if cabinet_type == 'remotecab':
                if x_offset + cylinder_height <= cab_size: # for outer cylinder
                    x_offset_inside += cylinder_height
            elif cabinet_type == 'hoodcab':
                if current_y_position + cylinder_height <= cab_size:
                    current_y_position += cylinder_height

        elif(tank_type.lower() == 'ansul'):
            draw_plan_view_ansul_cylinder(
                cabinet_block, 
                x_offset, 
                current_y_position_outside,
                close=close, 
                dxfattribs=dxfattribs, 
                cab_location=cab_location,
                cab_width=cab_width, 
                cylinder_radius = cylinder_radius,
                outer_cylinder = outer_cylinder,
                outer_cylinder_qty = outer_cylinder_qty,
                add_outer_label = add_outer_label,
                cabinet_type= cabinet_type
            )

            if cabinet_type == 'remotecab':
                if x_offset + cylinder_height <= cab_size: # for inner cylinder
                    x_offset_inside += cylinder_height
                    current_y_position_outside -= 1.6

            if outer_cylinder:
                add_outer_label = outer_cylider_idx > 1
                outer_cylider_idx += 1
                if cabinet_type == 'hoodcab':
                    if(cab_location.lower() == 'left'):
                        if tank_qty - (cylinder + 1) > 0:
                            outer_cylinder_separator_points = [
                                (x_offset - cylinder_radius - 0.5, current_y_position_outside + cylinder_radius),
                                (x_offset - cylinder_radius - 0.5, current_y_position_outside - cylinder_radius),
                            ]
                            cabinet_block.add_lwpolyline(outer_cylinder_separator_points, close=True, dxfattribs=dxfattribs)
                        
                        if outer_cylinder and actuator_count == 0:
                            actuator_count += 1
                            actuator_points = [
                                (x_offset - (((cylinder_radius * 2) + 0.3) * outer_cylinder_qty) + 2 - 0.5, current_y_position_outside + cylinder_radius - 0.5),
                                (x_offset - (((cylinder_radius * 2) + 0.3) * outer_cylinder_qty) + 2 - 0.5, current_y_position_outside - cylinder_radius + 1),
                                (x_offset - (((cylinder_radius * 2) + 0.3) * outer_cylinder_qty) + 2 - 6.5, current_y_position_outside - cylinder_radius + 1),
                                (x_offset - (((cylinder_radius * 2) + 0.3) * outer_cylinder_qty) + 2 - 6.5, current_y_position_outside + cylinder_radius - 0.5),
                            ]
                            cabinet_block.add_lwpolyline(actuator_points, close=True, dxfattribs=dxfattribs)
                            
                            outer_cylinder_hood_points = [
                                (x_offset + cylinder_radius + 0.5, current_y_position_outside - cylinder_radius),
                                (x_offset + cylinder_radius + 0.5, current_y_position_outside + cylinder_radius),
                                (x_offset - ((cylinder_radius * 2) * outer_cylinder_qty) - 6, current_y_position_outside + cylinder_radius),
                                (x_offset - ((cylinder_radius * 2) * outer_cylinder_qty) - 6, current_y_position_outside - cylinder_radius),
                            ]
                            cabinet_block.add_lwpolyline(outer_cylinder_hood_points, close=True, dxfattribs=dxfattribs)
                    else:
                        if tank_qty - (cylinder + 1) > 0:
                            outer_cylinder_separator_points = [
                                (x_offset + cylinder_radius + 0.5, current_y_position_outside + cylinder_radius),
                                (x_offset + cylinder_radius + 0.5, current_y_position_outside - cylinder_radius),
                            ]
                            cabinet_block.add_lwpolyline(outer_cylinder_separator_points, close=True, dxfattribs=dxfattribs)
                
                        if outer_cylinder and actuator_count == 0:
                            actuator_count += 1
                            actuator_points = [
                                (x_offset + (((cylinder_radius * 2) + 0.3) * outer_cylinder_qty) - 2 + 0.5, current_y_position_outside + cylinder_radius - 0.5),
                                (x_offset + (((cylinder_radius * 2) + 0.3) * outer_cylinder_qty) - 2 + 0.5, current_y_position_outside - cylinder_radius + 1),
                                (x_offset + (((cylinder_radius * 2) + 0.3) * outer_cylinder_qty) - 2 + 6.5, current_y_position_outside - cylinder_radius + 1),
                                (x_offset + (((cylinder_radius * 2) + 0.3) * outer_cylinder_qty) - 2 + 6.5, current_y_position_outside + cylinder_radius - 0.5),
                            ]
                            cabinet_block.add_lwpolyline(actuator_points, close=True, dxfattribs=dxfattribs)
                            
                            
                            outer_cylinder_hood_points = [
                                (x_offset - cylinder_radius - 0.5, current_y_position_outside - cylinder_radius),
                                (x_offset - cylinder_radius - 0.5, current_y_position_outside + cylinder_radius),
                                (x_offset + ((cylinder_radius * 2) * outer_cylinder_qty) + 6, current_y_position_outside + cylinder_radius),
                                (x_offset + ((cylinder_radius * 2) * outer_cylinder_qty) + 6, current_y_position_outside - cylinder_radius),
                            ]
                            cabinet_block.add_lwpolyline(outer_cylinder_hood_points, close=True, dxfattribs=dxfattribs)

                elif cabinet_type == 'remotecab':
                    if tank_qty - (cylinder + 1) > 0:
                        outer_cylinder_separator_points = [
                            (x_offset - cylinder_radius, current_y_position_outside + cylinder_radius + 0.5),
                            (x_offset + cylinder_radius , current_y_position_outside + cylinder_radius + 0.5),
                        ]
                        cabinet_block.add_lwpolyline(outer_cylinder_separator_points, close=True, dxfattribs=dxfattribs)
            
                    if outer_cylinder and actuator_count == 0:
                        actuator_count += 1
                        actuator_points = [
                            (x_offset + cylinder_radius - 0.5, current_y_position_outside - (((cylinder_radius * 2) + 0.3) * outer_cylinder_qty) - 2 - 0.5),
                            (x_offset - cylinder_radius + 1, current_y_position_outside - (((cylinder_radius * 2) + 0.3) * outer_cylinder_qty) - 2 - 0.5),
                            (x_offset - cylinder_radius + 1, current_y_position_outside - (((cylinder_radius * 2) + 0.3) * outer_cylinder_qty) - 2 - 6.5),
                            (x_offset + cylinder_radius - 0.5, current_y_position_outside - (((cylinder_radius * 2) + 0.3) * outer_cylinder_qty) - 2 - 6.5),
                        ]
                        cabinet_block.add_lwpolyline(actuator_points, close=True, dxfattribs=dxfattribs)

                        actuator_third_point = actuator_points[2]
                        actuator_fourth_point = actuator_points[3]
                        outer_cylinder_hood_points = [
                            (x_offset - cylinder_radius, current_y_position_outside + cylinder_radius + 0.5),
                            (x_offset + cylinder_radius, current_y_position_outside + cylinder_radius + 0.5),
                            (x_offset + cylinder_radius, actuator_third_point[1] - 2),
                            (x_offset - cylinder_radius, actuator_fourth_point[1] - 2),
                        ]
                        cabinet_block.add_lwpolyline(outer_cylinder_hood_points, close=True, dxfattribs=dxfattribs)
                        outer_cylinder_hood_points_bottom =  outer_cylinder_hood_points[3]

                if cabinet_type == 'remotecab' and outer_cylinder:
                    draw_plan_view_info_label(
                        cabinet_block,
                        start_x_offset=x_offset + 25,
                        start_y_offset=outer_cylinder_hood_points_bottom[1] / 2,
                        pointer_x_offset=x_offset + 10,
                        pointer_y_offset=outer_cylinder_hood_points_bottom[1] / 2 - 0.5,
                        alignment='left',
                        text=f'ANSUL REGULATED \nACTUATOR AND \nSINGLE TANK \nENCLOSURE WITH \n ({outer_cylinder_qty}) 3GAL \nCYLINDER.'
                    )

        if cabinet_type == 'hoodcab':
            if current_y_position + cylinder_height <= cab_size:
                current_y_position += cylinder_height
   
    if(cabinet_type == 'remotecab'):
        draw_remote_cabinet_info_label(
            cabinet_block, 
            text=f'FIRE CABINET \n W/ACCESS DOOR',
            start_x_offset = control_type + 0.5, start_y_offset = 0 - cab_width - 8, 
            pointer_x_offset = control_type + 0.5, pointer_y_offset = 0 - cab_width,
            alignment='bottom'
        )
    if(cabinet_type == 'hoodcab'):
        if(cab_location.lower() == 'left'):
            draw_plan_view_info_label(
                cabinet_block, 
                start_x_offset = start_x_offset,
                start_y_offset = cab_bottom_right[1] + control_type + 1 + 0.5,
                pointer_x_offset = pointer_x_offset - 1,
                pointer_y_offset = cab_bottom_right[1] + control_type + 1,
                alignment='right',
                text='FIRE CABINET \n W/ACCESS DOOR',
            )
        else:
            draw_plan_view_info_label(
                cabinet_block, 
                start_x_offset = start_x_offset,
                start_y_offset = cab_bottom_right[1] + control_type + 1 + 0.5,
                pointer_x_offset = pointer_x_offset + 1,
                pointer_y_offset = cab_bottom_right[1] + control_type + 1,
                alignment='left',
                text='FIRE CABINET \n W/ACCESS DOOR',
            )

    # CABINET COMPONENT OUTLINE

    if cabinet_type == 'remotecab':
        x_offset_cab = 0
        y_offset_cab = 0
        control_box_outline_points = [
            (x_offset_cab, y_offset_cab - cab_width),
            (x_offset_cab + cab_size, y_offset_cab - cab_width),
            (x_offset_cab + cab_size, y_offset_cab - cab_width - 40),
            (x_offset_cab, y_offset_cab - cab_width - 40),
        ]
        cabinet_block.add_lwpolyline(control_box_outline_points, close=True, dxfattribs={'layer': 'CONTROL-BOX-LINES'})

        cabinet_block.add_mtext(
            '36" CLEARANCE', 
            dxfattribs={
                'layer': 'CONTROL-BOX-LINES',
                'insert': ((cab_size / 2) / 2,  y_offset_cab + 5),
                'char_height': 2,
            }
        )
    elif cabinet_type == 'hoodcab':
        label_x_offset = 0
        label_y_offset = 0
        if(cab_location.lower() == 'left'):
            control_box_outline_points = [
                (0, cab_top_left[1]),
                (0 - 36, cab_top_left[1]),
                (0 - 36, cab_top_left[1] - cab_size),
                (0, cab_top_left[1] - cab_size),
            ]
            label_x_offset = -36 + 5
        else: 
            control_box_outline_points = [
                (cab_width, cab_top_left[1]),
                (cab_width + 36, cab_top_left[1]),
                (cab_width + 36, cab_top_left[1] - cab_size),
                (cab_width, cab_top_left[1] - cab_size),
            ]
            label_x_offset = cab_width + 5
        cabinet_block.add_lwpolyline(control_box_outline_points, close=True, dxfattribs={'layer': 'CONTROL-BOX-LINES'})

        cabinet_block.add_mtext(
            '36" CLEARANCE', 
            dxfattribs={
                'layer': 'CONTROL-BOX-LINES',
                'insert': (label_x_offset, cab_size + 5),
                'char_height': 2,
            }
        )
    


def draw_plan_view_cabinet(msp, cab_bottom_left, cab_bottom_right, cab_top_right, cab_top_left, close=True, dxfattribs={'layer': 'HOOD'}, cabinet={}):
    
    cabinet_block = cabinet['cabinet_block'] 
    control_type = cabinet['control_type']
    cab_width = cabinet['cab_width']
    cab_size = cabinet['cab_size']
    cab_location = cabinet['cab_location']

    control_portion_type = cabinet['control_portion_type']
    cabinet_type = cabinet['cabinet_type']

    logger.debug(f'control_portion_type: {control_portion_type}')
    logger.debug(f'cabinet_type: {cabinet_type}')

    if control_portion_type == 'dcv':
        control_portion_label = f'TOUCHPAD DEMAND\n VENDILATION CONTROL'
    elif control_portion_type == 'std':
        control_portion_label = f'SWITCH PANEL\n1 LIGHT, 1 FAN'

    fire_portion_size = control_type - cab_size

    cabinet_center_x = (cab_bottom_left[0] + cab_bottom_right[0]) / 2
    cabinet_x = cab_bottom_left[0]
    cabinet_y = cab_bottom_left[1]
    
    if cabinet_type == 'remotecab':
            draw_remote_cabinet_info_label(
                msp, 
                text=control_portion_label,
                start_x_offset = cabinet_x + control_type/2, start_y_offset=cabinet_y - 6, 
                pointer_x_offset = cabinet_x + control_type/2, pointer_y_offset=cab_bottom_left[1],
                alignment='bottom'
            )
    if cabinet_type == 'hoodcab':
        if(cab_location.lower() == 'left'):
            draw_plan_view_info_label(
                msp, 
                text=control_portion_label, 
                start_x_offset = cabinet_x - cab_width -15, start_y_offset=cabinet_y - 3, 
                pointer_x_offset = cab_bottom_left[0] - cab_width, pointer_y_offset=cab_bottom_left[1],
                alignment='right'
            )
        else:
            draw_plan_view_info_label(
                msp, 
                text=control_portion_label, 
                start_x_offset = cabinet_x + 15, start_y_offset=cabinet_y - 3, 
                pointer_x_offset = cab_bottom_left[0], pointer_y_offset=cab_bottom_left[1],
                alignment='left'
            )

    draw_plan_view_control_portion(cabinet_block, control_type, cab_width, cab_location, control_portion_type, cabinet_type, cab_bottom_left, cab_bottom_right, cab_top_right, cab_top_left, dxfattribs={'layer': 'HOOD ACCESSORIES'}, text="Control")

    draw_plan_view_fire_portion(cab_size, fire_portion_size, control_type, cabinet_block, cab_bottom_left, cab_bottom_right, cab_top_right, cab_top_left, close=close, dxfattribs={'layer': 'FP COMPONENTS'}, cabinet=cabinet)
    
    msp.add_lwpolyline([cab_bottom_left, cab_bottom_right, cab_top_right, cab_top_left], close=close, dxfattribs=dxfattribs)



# DRAWING DXF

def draw(form_data, number_of_hoods):

    logger.debug(f'form_data: {form_data}')
    logger.debug(f'number_of_hoods: {number_of_hoods}')

    input_params = form_data['InputParams']

    cab_left_cheek = input_params['CABINETLEFTCHEEK']
    cab_right_cheek = input_params['CABINETRIGHTCHEEK']
    plenum_depth = input_params['PLENUMDEPTH']
    plenum_length = input_params['PLENUMLENGTH']
    tapper_from_top = input_params['TAPERFROMTOP']
    tapper_from_back = input_params['TAPERFROMBACK']
    heat_sensor = input_params['HEATSENSOR']
    top_of_hood = input_params['TOPOFHOOD']
    hood_front = input_params['HOODFRONT']
    hood_back = input_params['HOODBACK']
    visibility1 = input_params['VISIBILITY1']
    cab_location = input_params['CABINETLOCATION']
    front_elevation_json = input_params['FrontElevation']
    section_view_json = input_params['SectionView']
    hood_info_json = []
    hood_schedule_info_json = []
    hood_schedule_table = []
    cabinet_info_json = []
    plenum_schedule_json = []
    fan_schedule_json = []


    doc = ezdxf.new(dxfversion='R2010')
    msp = doc.modelspace()

    # LAYERS

    doc.linetypes.add(name="DASHED_CUSTOM", pattern="A, 6.0, -2.0")
    doc.linetypes.add(name="ZERO_CLEARANCE", pattern="A, 1.5, -1.5")

    doc.layers.add(name="HOOD", color=colors.RED, linetype="DASHED", lineweight=50)
    doc.layers.add(name="HOOD ACCESSORIES", color=colors.YELLOW, linetype="DASHED")
    doc.layers.add(name="EXHAUST-DUCT", color=colors.GREEN, linetype="DASHED")
    doc.layers.add(name="HOOD ELECTRICAL", color=colors.BLUE, linetype="DASHED")
    doc.layers.add(name="TEXT", color=colors.YELLOW)
    doc.layers.add(name='DIM', color=colors.GRAY, linetype='DASHED')
    doc.layers.add(name='MUA-DUCT', color=colors.MAGENTA, linetype='DASHED')
    doc.layers.add(name='HOOD-ACCESSORIES', color=colors.YELLOW, linetype='DASHED')
    doc.layers.add(name='FLOW-INDICATOR', color=colors.RED, linetype='DASHED')
    doc.layers.add(name='INDICATOR', color=colors.CYAN, linetype='DASHED')
    doc.layers.add(name='HOOD-LABEL', color=colors.YELLOW, linetype='DASHED')
    doc.layers.add(name='APPLIANCE', color=colors.BYLAYER, linetype='DASHED')
    doc.layers.add(name='TABLE', color=colors.BYLAYER, linetype='DASHED')
    doc.layers.add(name="CONTROL-BOX-LINES", color=colors.YELLOW, linetype="DASHED_CUSTOM")
    doc.layers.add(name='FP COMPONENTS', color=colors.WHITE)
    doc.layers.add(name="ZERO CLEARANCE", color=colors.WHITE, linetype="ZERO_CLEARANCE")


    x_offset = 10
    start_x_offset = x_offset
    y_offset = 0
    is_first_iteration = True

    for hood_index  in range(number_of_hoods):
        hood_information = input_params['HOOD_INFORMATION'][hood_index]
        exh_collar_information_list = input_params['HOOD_INFORMATION'][hood_index]['EXH_COLLAR_LOCATION_INFORMATION']
        mua_collar_information_list = input_params['HOOD_INFORMATION'][hood_index]['MUA_COLLAR_LOCATION_INFORMATION']
        cabinet_information = input_params['CABINET_INFORMATION'][hood_index]
        hood_schedule_information = input_params['HOOD_SCHEDULE_INFO'][hood_index]
        plenum_schedule_information = input_params['PLENUM_SCHEDULE_INFO'][hood_index]
        fan_schedule_information = input_params['FAN_SCHEDULE_INFO'][hood_index]

        
        #HOOD INFORMATION

        hood_length = hood_information['HOODLENGTH']
        hood_width = hood_information['HOODWIDTH']
        hood_height = hood_information['HOODHEIGHT']
        hang_height = hood_information['HANGHEIGHT']
        hood_plan = hood_information['HOOD_PLAN']
        hood_elev = hood_information['HOOD_ELEV']
        hood_sec = hood_information['HOOD_SECT']
        drain = hood_information['DRAIN']
        hd_right_cheek = hood_information['RIGHTCHEEK']
        hd_left_cheek = hood_information['LEFTCHEEK']
        
        exh_collar_qty = hood_schedule_information['EXH_COLLAR_QTY']
        mua_collar_qty = plenum_schedule_information['MUA_COLLAR_QTY']
        
        hood_information['EXH_COLLAR_QTY'] = exh_collar_qty
        hood_information['MUA_COLLAR_QTY'] = mua_collar_qty
        hood_information['EXH_COLLAR'] = exh_collar_information_list[0]
        hood_information['MUA_COLLAR'] = mua_collar_information_list[0]
        
        
        # EXH COLLAR LOCATION INFORMATION

        for exh_collar_information in exh_collar_information_list:
            exh_collar_length = exh_collar_information['EXHCOLLARLENGTH']
            exh_collar_width = exh_collar_information['EXHCOLLARWIDTH']
            exh_collar_loc = exh_collar_information['EXHCOLLARLOCATION']
            hood_information['EXH_COLLAR_SIZE'] = f"{exh_collar_length} X {exh_collar_width}"

        
        # MUA COLLAR LOCATION INFORMATION

        for mua_collar_information in mua_collar_information_list:
            mua_collar_length = mua_collar_information['MUACOLLARLENGTH']
            mua_collar_width = mua_collar_information['MUACOLLARWIDTH']
            mua_collar_loc = mua_collar_information['MUA_COLLAR_LOCATION']


        # CABINET INFORMATION

        cab_visibility = cabinet_information['CABINETVISIBILITY']
        cab_size = cabinet_information['CABINETSIZE']
        cab_height = cabinet_information['CABINETHEIGHT']
        cab_width = cabinet_information['CABINETWIDTH']
        remote_cab_type = cabinet_information['REMOTECABINETTYPE']
        control_type = cabinet_information['CONTROLTYPE']
        tank_qty = cabinet_information['TANKQTY']

        if(cab_location.lower() == 'left' and hood_index == 0):
            x_offset += cab_width

        cab_visibility = cab_visibility.lower()
        cab_visibility = [item.strip() for item in cab_visibility.split('-')]
        control_portion_type = cab_visibility[0]
        cabinet_type = cab_visibility[1]

        # HOOD SCHEDULE INFO

        hd_hood_no = hood_schedule_information['HOOD_NO']
        mfg = hood_schedule_information['MFG']
        model = hood_schedule_information['MODEL']
        hood_type = hood_schedule_information['HOOD_TYPE']
        material = hood_schedule_information['MATERIAL']
        hood_wgt = hood_schedule_information['HOOD_WGT']
        exh_collar_qty = hood_schedule_information['EXH_COLLAR_QTY']
        exh_collar_sp = hood_schedule_information['EXH_COLLAR_SP']
        exh_collar_area = hood_schedule_information['EXH_COLLAR_AREA']
        exh_collar_cfm = hood_schedule_information['EXH_COLLAR_CFM']
        grease_collar_vel = hood_schedule_information['GREASE_COLLAR_VELOCITY']
        filter_type = hood_schedule_information['FILTER_TYPE']
        filter_qty1 = hood_schedule_information['FILTER_QTY1']
        filter_size1 = hood_schedule_information['FILTER_SIZE1']
        filter_qty2 = hood_schedule_information['FILTER_QTY2']
        filter_size2 = hood_schedule_information['FILTER_SIZE2']
        filter_area = hood_schedule_information['FILTER_AREA']
        filter_vel = hood_schedule_information['FILTER_VELOCITY']
        light_qty = hood_schedule_information['LIGHT_QTY']
        light_type = hood_schedule_information['LIGHT_TYPE']
        switch_light = hood_schedule_information['SWITCH_LIGHT']


        # PLENUM SCHEDULE INFO

        pl_hood_no = plenum_schedule_information['HOOD_NO']
        mua_collar_qty = plenum_schedule_information['MUA_COLLAR_QTY']
        collar_sp = plenum_schedule_information['COLLAR_SP']
        collar_cfm = plenum_schedule_information['COLLAR_CFM']
        mua_area = plenum_schedule_information['MUA_AREA']
        mua_vel = plenum_schedule_information['MUA_VELOCITY']
        plenum_height = plenum_schedule_information['PLENUM_WGT']

        hood_information['HOOD_WGT'] = hood_wgt
        hood_info_json.append(hood_information)
        cabinet_info_json.append(cabinet_information)
        hood_schedule_info_json.append(hood_schedule_information)
        plenum_schedule_json.append(plenum_schedule_information)
        fan_schedule_json.append(fan_schedule_information)


        # DRAW HOOD

        hood_bottom_left = (x_offset, 0)
        hood_bottom_right = (x_offset + hood_length, 0)
        hood_top_right = (x_offset + hood_length, hood_width)
        hood_top_left = (x_offset, hood_width)
        points = [hood_bottom_left, hood_bottom_right, hood_top_right, hood_top_left]
        msp.add_lwpolyline(points, close=True, dxfattribs={'layer': 'HOOD'})
        hood_center_x = hood_length / 2
        hood_center_y = hood_width / 2

        # ADD HOOD NAME

        hood_name_pos_x = hood_bottom_left[0] + 7
        hood_name_pos_y = hood_bottom_left[1] + 3

        msp.add_text(
            f"HOOD #{hd_hood_no}", 
            dxfattribs={
                'layer': 'TEXT',
                'height': 2.0,
                'style': 'STANDARD_FONT',
            }
        ).set_placement((hood_name_pos_x, hood_name_pos_y), align=TextEntityAlignment.LEFT)


        # HOOD RIGHT CHEECK

        rc_text = None

        cont_capture_text = "CONT. CAPTURE"
        zero_clearance_text = "0\" CLEARANCE"
        finished_cheek_text = "FINISH"

        if hd_left_cheek.lower() == '0 clearance':
            zcl_start_point = (x_offset + hood_length - 1, y_offset)
            zcl_end_point = (x_offset + hood_length - 1, hood_width)
            msp.add_line(start=zcl_start_point, end=zcl_end_point, dxfattribs={'layer': 'ZERO CLEARANCE'})
        
        if not is_first_iteration and not cab_location.lower() == 'right':
            rc_text = finished_cheek_text
        elif not is_first_iteration and cab_location.lower() == 'right' and cabinet_type == 'remotecab':
            rc_text = finished_cheek_text
        elif not is_first_iteration and cab_location.lower() == 'right' and cabinet_type == 'hoodacb':
            rc_text = control_portion_type.upper()
        else:
            rc_text = cont_capture_text

        rc_text_height = 1
        rc_text_width = get_text_width(rc_text, rc_text_height)

        rc_text_x = hood_bottom_right[0] - 1.5
        rc_text_y = hood_bottom_right[1] + (hood_width / 2) - (rc_text_width / 2)
            
        rc_text_entity = msp.add_text(
            rc_text,
            dxfattribs={
                'layer': 'TEXT',
                'height': rc_text_height,
                'style': 'STANDARD_FONT',
                'rotation': 90,
                'insert': (rc_text_x, rc_text_y)
            }
        )

        rc_start_point = (rc_text_x + 0.65, rc_text_y - 1)
        rc_end_point = (rc_text_x + 0.65, rc_text_y + rc_text_width)

        msp.add_line(rc_start_point, rc_end_point, dxfattribs={'color': colors.RED})

        # HOOD LEFT CHEECK

        lc_text = None

        cont_capture_text = "CONT. CAPTURE"
        zero_clearance_text = "0\" CLEARANCE"
        finished_cheek_text = "FINISH"

        if hd_left_cheek.lower() == '0 clearance':
            zcl_start_point = (x_offset + 1, y_offset)
            zcl_end_point = (x_offset + 1, hood_width)
            msp.add_line(start=zcl_start_point, end=zcl_end_point, dxfattribs={'layer': 'ZERO CLEARANCE'})
        
        if is_first_iteration and not cab_location.lower() == 'left':
            lc_text = finished_cheek_text
        elif is_first_iteration and cab_location.lower() == 'left' and cabinet_type == 'remotecab':
            lc_text = finished_cheek_text
        elif is_first_iteration and cab_location.lower() == 'left' and cabinet_type == 'hoodacb':
            lc_text = control_portion_type.upper()
        else:
            lc_text = cont_capture_text

        lc_text_height = 1
        lc_text_width = get_text_width(lc_text, lc_text_height)

        lc_text_x = hood_bottom_left[0] + 1.45
        lc_text_y = hood_bottom_left[1] + (hood_width / 2) + (lc_text_width / 2)
            
        if hd_left_cheek:
            lc_text_entity = msp.add_text(
                lc_text,
                dxfattribs={
                    'layer': 'TEXT',
                    'height': lc_text_height,
                    'style': 'STANDARD_FONT',
                    'rotation': 270,
                    'insert': (lc_text_x, lc_text_y)
                }
            )

        lc_start_point = (lc_text_x - 0.65, lc_text_y)
        lc_end_point = (lc_text_x - 0.65, lc_text_y - lc_text_width)

        msp.add_line(lc_start_point, lc_end_point, dxfattribs={'color': colors.RED})


        # HOOD LINE

        hood_line_y = 0.9 * hood_width
        start_point = (x_offset, hood_line_y)
        end_point = (x_offset + hood_length, hood_line_y)
        msp.add_line(start=start_point, end=end_point, dxfattribs={'layer': 'HOOD'})


        # ZERO CLEARANCE

        if top_of_hood.lower() == '0 clearance':
            zero_clearance_line = hood_width - 1
            zc_start_point = (x_offset, zero_clearance_line)
            zc_end_point = (x_offset + hood_length, zero_clearance_line)
            msp.add_line(start=zc_start_point, end=zc_end_point, dxfattribs={'layer': 'ZERO CLEARANCE'})


        # HOOD ACCESSORIES

        small_rect_width = 4
        small_rect_height = 2
        small_circle_radius = 0.3

        left_rect_bottom_left = (hood_bottom_left[0] + 1, hood_line_y - 1.5 * small_rect_height)
        right_rect_bottom_left = (hood_bottom_right[0] - 1 - small_rect_width, hood_line_y - 1.5 * small_rect_height)
        add_rectangle_with_circles(msp, left_rect_bottom_left, small_rect_width, small_rect_height, small_circle_radius, layer='HOOD ACCESSORIES')
        add_rectangle_with_circles(msp, right_rect_bottom_left, small_rect_width, small_rect_height, small_circle_radius, layer='HOOD ACCESSORIES')

        small_triangle_width = 4
        small_triangle_height = 2

        triangle_bottom_left_start = (hood_bottom_left[0] + 1, hood_bottom_left[1] + 1)
        triangle_bottom_left_end = (hood_bottom_left[0] + 1 + small_triangle_width, hood_bottom_left[1] + 1)
        triangle_bottom_left_vertext = (hood_bottom_left[0] + 1, hood_bottom_left[1] + 1 + small_triangle_height)
        triangle_bottom_right_start = (hood_bottom_right[0] - 1, hood_bottom_right[1] + 1)
        triangle_bottom_right_end = (hood_bottom_right[0] - 1 - small_triangle_width, hood_bottom_right[1] + 1)
        triangle_bottom_right_vertext = (hood_bottom_right[0] - 1, hood_bottom_right[1] + 1 + small_triangle_height)
        add_triangle(msp, triangle_bottom_left_start, triangle_bottom_left_end, triangle_bottom_left_vertext, small_circle_radius, layer='HOOD ACCESSORIES')
        add_triangle(msp, triangle_bottom_right_start, triangle_bottom_right_end, triangle_bottom_right_vertext, small_circle_radius, layer='HOOD ACCESSORIES')


        # HOOD COLLARS

        hood_collar_top_y = 0.85 * hood_width
        total_collars_width = exh_collar_length * exh_collar_qty
        total_gaps_width = hood_length - total_collars_width
        gap_between_collars = total_gaps_width / (exh_collar_qty + 1)
        segment_length = hood_length / exh_collar_qty
        hood_collar_bottom_y = None

        for exh_collar_index in range(exh_collar_qty):
            segment_start_x = hood_bottom_left[0] + exh_collar_index * segment_length
            start_x = segment_start_x + (segment_length - exh_collar_length) / 2
            bottom_left_x = start_x
            bottom_left_y = hood_collar_top_y - exh_collar_width
            hood_collar_bottom_y = hood_collar_top_y - exh_collar_width

            hood_collar_bottom_left = (bottom_left_x, bottom_left_y)
            hood_collar_bottom_right = (bottom_left_x + exh_collar_length, bottom_left_y)
            hood_collar_top_right = (bottom_left_x + exh_collar_length, hood_collar_top_y)
            hood_collar_top_left = (bottom_left_x, hood_collar_top_y)
            hood_collar_points = [hood_collar_bottom_left, hood_collar_bottom_right, hood_collar_top_right, hood_collar_top_left]
            
            msp.add_lwpolyline(hood_collar_points, close=True, dxfattribs={'layer': 'EXHAUST-DUCT'})
            msp.add_line(start=hood_collar_bottom_left, end=hood_collar_top_right, dxfattribs={'layer': 'EXHAUST-DUCT'})
            
            width_text_x = bottom_left_x + 2
            width_text_y = (bottom_left_y + hood_collar_top_y) / 2
            
            length_text_x = (bottom_left_x + hood_collar_bottom_right[0]) / 2
            length_text_y = hood_collar_top_y - 3
            
            width_text_width = get_text_width(f"{exh_collar_width}", 2)
            length_text_width = get_text_width(f"{exh_collar_length}", 2)
            text_height = 2
            
            width_text_y = width_text_y - (text_height / 2)
            
            length_text_x = length_text_x - (length_text_width / 2)
            
            # ADD EXC COLLAR WIDTH
            msp.add_text(
                text=f"{exh_collar_width}",
                dxfattribs={
                    'insert': (width_text_x, width_text_y),
                    'layer': 'TEXT',
                    'height': text_height,
                }
            )

            # ADD EXC COLLAR LENGTH
            msp.add_text(
                text=f"{exh_collar_length}",
                dxfattribs={
                    'insert': (length_text_x, length_text_y),
                    'layer': 'TEXT',
                    'height': text_height,
                }
            )
            if(hood_index == 0):
                draw_horizontal_dimension_line_left(
                    msp, 
                    bottom_left_x, bottom_left_y + 30, 
                    height=exh_collar_length, dist=35, 
                    val=f"11\" EXHAUST ?", 
                    layer='EXHAUST-DUCT'
                )


        # HEAT SENSOR

        if heat_sensor.lower() == 'yes':
            sensor_qty = 1
            total_sensor_length = 0.9 * hood_length
            sensor_width = 4
            sensor_height = 2
            gap_length = (total_sensor_length - sensor_qty * sensor_width) / (sensor_qty + 1)
            sensors_start_x = x_offset + hood_center_x - total_sensor_length / 2
            sensor_center_y = 0.42 * hood_width

            sensor_center_x = sensors_start_x + gap_length + sensor_width / 2
            left_edge, right_edge, sensor_top_right_y, sensor_width = draw_sensor(msp, sensor_center_x, sensor_center_y, sensor_width, sensor_height)

            text_height = 1.5
            vertical_offset = 0.5

            text = "HEAT SENSOR"
            text_width = get_text_width(text, text_height)

            text_x = x_offset + hood_center_x - text_width / 2
            text_y = sensor_top_right_y + vertical_offset + text_height / 2

            msp.add_text(
                text=text,
                dxfattribs={
                    'insert': (text_x, text_y),
                    'layer': 'TEXT',
                    'height': text_height
                }
            )


        # LIGHTS

        lights_total_length = 0.9 * hood_length
        light_height = light_width = 4
        gap_length = (lights_total_length - light_qty * light_width) / (light_qty + 1)
        lights_start_x = x_offset + hood_center_x - lights_total_length / 2
        light_top_y = 0.35 * hood_width

        edges = []

        for light_index in range(light_qty):
            bottom_left_x = lights_start_x + light_index * (light_width + gap_length) + gap_length
            bottom_left_y = light_top_y - light_height
            bottom_right_x, bottom_right_y = draw_lights_with_circle(msp, bottom_left_x, bottom_left_y, light_width, light_height)
            edges.append((bottom_right_x, bottom_right_y + light_height / 2))

        offset = light_height / 8

        for lgt_edge_index in range(len(edges) - 1):
            start_point = edges[lgt_edge_index]
            end_point = (edges[lgt_edge_index + 1][0] - light_width, edges[lgt_edge_index + 1][1])
            center_y = (start_point[1] + end_point[1]) / 2
            msp.add_line(
                (start_point[0], center_y + offset),
                (end_point[0], center_y + offset),
                dxfattribs={'layer': 'HOOD ELECTRICAL'}
            )
            msp.add_line(
                (start_point[0], center_y - offset),
                (end_point[0], center_y - offset),
                dxfattribs={'layer': 'HOOD ELECTRICAL'}
            )


        # PLENUM

        plenum_length = hood_length
        plenum_start = hood_bottom_left

        #if cabinet_type == 'hoodcab':

        if(cab_location.lower() == 'left' and cabinet_type == 'hoodcab'):
            if(hood_index == 0):
                plenum_start = (plenum_start[0] - cab_width, plenum_start[1])
            else:
                plenum_start = (plenum_start[0], plenum_start[1])

        if hood_index == number_of_hoods - 1 and cab_location.lower() == 'right' and cabinet_type == 'hoodcab':
            plenum_length = hood_length + cab_width

        if(cab_location.lower() == 'left' and hood_index == 0 and cabinet_type == 'hoodcab'):
            plenum_length = hood_length + cab_width

        plenum_top_left = plenum_start
        plenum_bottom_left = (plenum_start[0], plenum_start[1] - plenum_depth)
        plenum_bottom_right = (plenum_start[0] + plenum_length, plenum_start[1] - plenum_depth)
        plenum_top_right = (plenum_start[0] + plenum_length, plenum_start[1])
        
        if(cab_location.lower() == 'left'):
            plenum_bottom_left = (plenum_start[0], plenum_start[1] - plenum_depth)
            plenum_bottom_right = (plenum_start[0] + plenum_length, plenum_start[1] - plenum_depth)
            plenum_top_right = (plenum_start[0] + plenum_length, plenum_start[1])

        plenum_points = [plenum_bottom_left, plenum_bottom_right, plenum_top_right, plenum_top_left]
        msp.add_lwpolyline(plenum_points, close=True, dxfattribs={'layer': 'HOOD'})

        # PLENUM COLLARS

        segment_width = plenum_length / mua_collar_qty
        center_y = (plenum_top_left[1] + plenum_bottom_left[1]) / 2
        first_collar_x = None

        current_x_offset = plenum_top_left[0]
        current_y_offset = center_y

        current_dim_x_offset = x_offset
        current_dim_y_offset = y_offset

        for pln_collar_index  in range(mua_collar_qty):
            segment_center_x = plenum_top_left[0] + (pln_collar_index * segment_width) + (segment_width / 2)
            
            bottom_left_x = segment_center_x - (mua_collar_length / 2)
            bottom_left_y = center_y - mua_collar_width / 2

            if pln_collar_index == 0:
                first_collar_x = bottom_left_x

            draw_mua_collars(msp, bottom_left_x, bottom_left_y, mua_collar_length, mua_collar_width, layer='MUA-DUCT')

            top_right_x = bottom_left_x + mua_collar_length
            top_right_y = bottom_left_y + mua_collar_width

            msp.add_line(
                (bottom_left_x, bottom_left_y),
                (top_right_x, top_right_y),
                dxfattribs={'layer': 'MUA-DUCT'}
            )
            msp.add_line(
                (bottom_left_x, top_right_y),
                (top_right_x, bottom_left_y),
                dxfattribs={'layer': 'MUA-DUCT'}
            )

            width_text_x = bottom_left_x + 2
            width_text_y = (bottom_left_y + top_right_y) / 2
            
            length_text_x = (bottom_left_x + top_right_x) / 2
            length_text_y = top_right_y - 3
            
            text_height = 2
            width_text_width = get_text_width(f"{mua_collar_width}", text_height)
            length_text_width = get_text_width(f"{mua_collar_length}", text_height)
            
            width_text_y = width_text_y - (text_height / 2)
            
            length_text_x = length_text_x - (length_text_width / 2)
            
            # ADD MUA COLLAR WIDTH
            msp.add_text(
                text=f"{mua_collar_width}",
                dxfattribs={
                    'insert': (width_text_x, width_text_y),
                    'layer': 'TEXT',
                    'height': text_height,
                }
            )

            # ADD MUA COLLAR LENGTH
            msp.add_text(
                text=f"{mua_collar_length}",
                dxfattribs={
                    'insert': (length_text_x, length_text_y),
                    'layer': 'TEXT',
                    'height': text_height,
                }
            )


            # DRAW PLAN VIEW PLENUM COLLAR DIMENSIONS FOR EACH COLLAR SEGMENT STARTS
            
            pln3_dim_hgt = 0

            current_y_offset = plenum_start[1] - plenum_depth

            for collar_index in range(mua_collar_qty + 1):

                if collar_index == 0:
                    current_collar_center_x = plenum_start[0]
                    next_collar_center_x = plenum_top_left[0] + (segment_width / 2)
                elif collar_index == mua_collar_qty:
                    current_collar_center_x = plenum_top_left[0] + ((collar_index - 1) * segment_width) + (segment_width / 2)
                    next_collar_center_x = plenum_top_right[0]
                else:
                    current_collar_center_x = plenum_top_left[0] + ((collar_index - 1) * segment_width) + (segment_width / 2)
                    next_collar_center_x = plenum_top_left[0] + (collar_index * segment_width) + (segment_width / 2)
                
                width_between_centers = next_collar_center_x - current_collar_center_x
                val_top = decimal_to_mixed_fraction(width_between_centers)

                draw_vertical_dimension_line(
                    msp,
                    current_collar_center_x,
                    current_y_offset,
                    height=pln3_dim_hgt,
                    width=width_between_centers,
                    val_top=val_top,
                    val_center="SUPPLY?",
                    layer='HOOD-ACCESSORIES'
                )

            # DRAW PLAN VIEW PLENUM COLLAR DIMENSIONS FOR EACH COLLAR SEGMENT ENDS


        # DRAW PLAN VIEW PLENUM DIMENSIONS / 2 STARTS

        #1st line... 
        pln2_dim_hgt = 25
        pln_collar_dim_width = hood_length / 2
        val_top = decimal_to_mixed_fraction(pln_collar_dim_width)
        draw_horizontal_dimension_line_plenum(
            msp,
            hood_collar_bottom_y, 
            current_dim_x_offset, current_dim_y_offset, 
            height=pln2_dim_hgt, 
            width=pln_collar_dim_width, 
            val_top=val_top, 
            val_center="EXHAUST ?", 
            layer='HOOD-ACCESSORIES'
        )

        #2nd line...
        second_dim_x_offset = current_dim_x_offset + pln_collar_dim_width
        val_top_second = decimal_to_mixed_fraction(pln_collar_dim_width)
        draw_horizontal_dimension_line_plenum(
            msp,
            hood_collar_bottom_y, 
            second_dim_x_offset, current_dim_y_offset, 
            height=pln2_dim_hgt, 
            width=pln_collar_dim_width, 
            val_top=val_top_second, 
            val_center="INTAKE ?", 
            layer='HOOD-ACCESSORIES'
        )

        # DRAW PLAN VIEW PLENUM DIMENSIONS / 2 ENDS

        
        # PLAN VIEW HOOD DIMENSIONS

        hd_dim_hgt = 35
        hd_dim_val_top = decimal_to_mixed_fraction(hood_length)
        hd_val_center = inches_to_feet_and_inches(hd_dim_val_top)

        draw_vertical_dimension_line(
            msp, 
            x_offset, y_offset, 
            height=hd_dim_hgt, width=hood_length, 
            val_top=f"{hd_dim_val_top}", val_center=hd_val_center, 
            layer='HOOD-ACCESSORIES'
        )

        current_dim_x_offset += pln_collar_dim_width

        # PLAN VIEW PLENUM DIMENSIONS

        pln_dim_hgt = 30
        pln_dim_val_top = decimal_to_mixed_fraction(plenum_length)
        pln_val_center = inches_to_feet_and_inches(pln_dim_val_top)

        current_x_offset = plenum_start[0]
        current_y_offset = plenum_start[1] - plenum_depth
        draw_vertical_dimension_line(
            msp, 
            current_x_offset, current_y_offset, 
            height=pln_dim_hgt, width=plenum_length, 
            val_top=f"{pln_dim_val_top}", val_center=pln_val_center, 
            layer='HOOD-ACCESSORIES'
        )
        
        if is_first_iteration:

            # HOOD HORIZONTAL DIMENSION
            hd_ver_dim_top = decimal_to_mixed_fraction(hood_width)
            hd_ver_dim_center = inches_to_feet_and_inches(hd_ver_dim_top)
            if(cab_location.lower() == 'left' and cabinet_type == 'hoodcab'):
                draw_horizontal_dimension_line_right(
                    msp, 
                    x_offset=x_offset + (hood_length * number_of_hoods), y_offset=y_offset + hood_width, 
                    height=hood_width, dist=45, 
                    val=f"{hd_ver_dim_top}\" {hd_ver_dim_center}", 
                    layer='HOOD-ACCESSORIES')
            else:
                draw_horizontal_dimension_line_left(msp, x_offset, y_offset + hood_width, hood_width, dist=45, val=f"{hd_ver_dim_top}\" {hd_ver_dim_center}", layer='HOOD-ACCESSORIES')

            # PLENUM HORIZONTAL DIMENSION
            pln_ver_dim_top = decimal_to_mixed_fraction(plenum_depth)
            pln_ver_dim_center = inches_to_feet_and_inches(pln_ver_dim_top)
            current_x_offset = plenum_start[0]
            current_y_offset = plenum_start[1]
            if(cab_location.lower() == 'left' and cabinet_type == 'hoodcab'):
                draw_horizontal_dimension_line_right(
                    msp, 
                    x_offset=x_offset + (hood_length * number_of_hoods), y_offset=current_y_offset, 
                    height=plenum_depth, dist=45, 
                    val=f"{pln_ver_dim_top}\" {pln_ver_dim_center}", 
                    layer='HOOD-ACCESSORIES'
                )
            else:
                draw_horizontal_dimension_line_left(
                    msp, 
                    x_offset=current_x_offset, y_offset=current_y_offset, 
                    height=plenum_depth, dist=45, 
                    val=f"{pln_ver_dim_top}\" {pln_ver_dim_center}", 
                    layer='HOOD-ACCESSORIES'
                )

            # PLENUM COLLAR VERTICAL DIMENSION
            pln_collar_ver_dim_top_str = decimal_to_mixed_fraction(plenum_depth/2)
            pln_collar_ver_dim_top = parse_mixed_fraction(pln_collar_ver_dim_top_str)
            pln_colla_ver_dim_center = inches_to_feet_and_inches(str(pln_collar_ver_dim_top))
            
            if(cab_location.lower() == 'left' and cabinet_type == 'hoodcab'):
                draw_vertical_dimension_line_plenum_right(
                    msp, 
                    bottom_right_x - 3, 
                    x_offset=x_offset + (hood_length * number_of_hoods), 
                    y_offset=current_y_offset - plenum_depth/2, 
                    height=plenum_depth/2, 
                    dist=25, 
                    val=f"{pln_collar_ver_dim_top_str}\" SUPPLY ?", 
                    layer='HOOD-ACCESSORIES',
                )
            elif(cab_location.lower() == 'right' and cabinet_type == 'hoodcab'):
                draw_vertical_dimension_line_plenum(
                    msp, 
                    first_collar_x, 
                    current_x_offset, current_y_offset - plenum_depth/2, 
                    height=plenum_depth/2, 
                    dist=25, 
                    val=f"{pln_collar_ver_dim_top_str}\" SUPPLY ?", 
                    layer='HOOD-ACCESSORIES',
                )
                
            is_first_iteration = False


        x_offset += hood_length


    # PLAN VIEW CABINET
    
    cabinet_block = doc.blocks.new(name='CABINET_BLOCK')

    cabinet_length = cab_width
    cabinet_width = hood_width 
    cabinet_height = hood_height

    if cabinet_type.lower() == 'remotecab':
        cab_bottom_left = (x_offset + 50, y_offset + hood_width - cab_width)
        cab_bottom_right = (x_offset + 50 + hood_width, y_offset + hood_width - cab_width)
        cab_top_right = (x_offset + 50 + hood_width, y_offset + hood_width)
        cab_top_left = (x_offset + 50, y_offset + hood_width)
    elif cabinet_type.lower() == 'hoodcab':
        if cab_location.lower() == 'right':
            cab_bottom_left = (x_offset + cabinet_length, y_offset)
            cab_bottom_right = (x_offset, y_offset)
            cab_top_right = (x_offset, y_offset + cabinet_width)
            cab_top_left = (x_offset + cabinet_length, y_offset + cabinet_width)
        elif cab_location.lower() == 'left':
            cab_bottom_left = (start_x_offset + cabinet_length, y_offset)
            cab_bottom_right = (start_x_offset, y_offset)
            cab_top_right = (start_x_offset, y_offset + cabinet_width)
            cab_top_left = (start_x_offset + cabinet_length, y_offset + cabinet_width)
    
    cab_location = cab_location.lower()

    draw_plan_view_cabinet(
        msp, 
        cab_bottom_left, cab_bottom_right, 
        cab_top_right, cab_top_left, 
        close=True, 
        dxfattribs={'layer': 'HOOD'}, 
        cabinet = {
            'cab_visibility': cab_visibility,
            'cab_location': cab_location,
            'cab_size': cabinet_width,
            'cab_width': cab_width,
            'cabinet_block': cabinet_block,
            'control_type': control_type,
            'tank_qty': tank_qty,
            'control_portion_type': control_portion_type,
            'cabinet_type': cabinet_type
        }
    )

    if cabinet_type == 'remotecab':
        msp.add_blockref('CABINET_BLOCK', (x_offset + 50, y_offset + hood_width), dxfattribs={'layer': 'HOOD ACCESSORIES', 'color': colors.YELLOW})
    elif cabinet_type == 'hoodcab':
        if(cab_location.lower() == 'left'):
            msp.add_blockref('CABINET_BLOCK', (start_x_offset, y_offset), dxfattribs={'layer': 'HOOD ACCESSORIES', 'color': colors.YELLOW})
        elif(cab_location.lower() == 'right'):
            msp.add_blockref('CABINET_BLOCK', (x_offset, y_offset), dxfattribs={'layer': 'HOOD ACCESSORIES', 'color': colors.YELLOW})


    # PLAN VIEW CABINET DIMENSION

    cab_dim_hgt = 35
    cab_dim_val_top = decimal_to_mixed_fraction(cabinet_length)
    cab_val_center = inches_to_feet_and_inches(cab_dim_val_top)
    
    if cabinet_type == 'hoodcab':
        draw_vertical_dimension_line(
            msp, 
            cab_bottom_right[0], cab_bottom_right[1], 
            height=cab_dim_hgt, width=cabinet_length, 
            val_top=f"{cab_dim_val_top}", val_center=cab_val_center, 
            layer='HOOD-ACCESSORIES',
            from_height=True
        )

    add_section_header(msp, text='PLAN VIEW', x_offset=hood_length/2, y_offset=y_offset - 100, font_height=6)

    # FRONT ELEVATION
    front_elevation_json['HOOD_INFORMATION'] = hood_info_json
    front_elevation_json['CABINET_INFORMATION'] = cabinet_info_json
    front_elevation_json['HOOD_SCHEDULE_INFO'] = hood_schedule_info_json
    hood_total_length = draw_front_elevation(front_elevation_json, doc, 0, y_offset - 300)
    
    # SECTION VIEW
    section_view_json['HOOD_INFORMATION'] = hood_info_json
    section_view_json['CABINET_INFORMATION'] = cabinet_info_json
    section_view_json['HOOD_SCHEDULE_INFO'] = hood_schedule_info_json
    draw_section_view(section_view_json, doc, hood_total_length + 150, y_offset - 300)
    
    # TABULAR VIEW
    hood_schedule_table = {
        "CABINETLOCATION": cab_location,
        "HOOD_INFORMATION": hood_info_json,
        "CABINET_INFORMATION": cabinet_info_json,
        "HOOD_SCHEDULE_INFO": hood_schedule_info_json
    }
    
    draw_hood_schedule_table(msp=msp, x_offset=-50, y_offset=y_offset - 400, number_of_hoods=number_of_hoods, table_data=hood_schedule_table)
    
    draw_plenum_schedule_table(msp=msp, x_offset=-50, y_offset=y_offset - 450, number_of_hoods=number_of_hoods, table_data=plenum_schedule_json)
    
    draw_fan_schedule_table(msp=msp, x_offset=160, y_offset=y_offset - 450, number_of_hoods=number_of_hoods, table_data=fan_schedule_json)

    file_path = "test.dxf"
    doc.saveas(file_path)
    return file_path