import math
from math import radians, cos, sin
import ezdxf
from ezdxf.math import Vec2
from ezdxf.render import mleader
from fractions import Fraction

from createdrawing.common import *


# DRAWING FRONT ELEVATION

def draw_accessories_tower_cap(msp, x_offset=0, y_offset=0, tower_width=0, tower_height=0, layer='HOOD-ACCESSORIES'):
    cap_points = [
        (x_offset, y_offset + tower_height),                           
        (x_offset + (- 0.3 * tower_width), y_offset + tower_height),       
        (x_offset + (0.3 * tower_width), y_offset + tower_height),       
        (x_offset + (0.45 * tower_width), y_offset + tower_height - 0.3 * tower_width),  
        (x_offset + (0.65 * tower_width), y_offset + tower_height + 0.02 * tower_height),
        (x_offset + (0.7 * tower_width), y_offset + tower_height),   
        (x_offset + (1.3 * tower_width), y_offset + tower_height)        
    ]
    msp.add_lwpolyline(cap_points, dxfattribs={'layer': layer})

def parse_mixed_fraction(mixed_str):
    if ' ' in mixed_str:
        whole_number, fraction = mixed_str.split(' ')
        numerator, denominator = fraction.split('/')
        return int(whole_number) * 12 + Fraction(int(numerator), int(denominator))
    elif '/' in mixed_str:
        numerator, denominator = mixed_str.split('/')
        return Fraction(int(numerator), int(denominator))
    else:
        return int(mixed_str)

def inches_to_feet_and_inches(mixed_str):
    inches = parse_mixed_fraction(mixed_str)
    feet = int(inches // 1) // 12
    remainder_inches = inches - feet * 12
    if remainder_inches == 0:
        return f"{feet}'-0\""
    else:
        return f"{feet}'-{remainder_inches:.0f}\""

def draw_hood_top_collar(msp, x_offset=0, y_offset=0, length=26, height=3, width=10, layer='HOOD-ACCESSORIES', flow_indicator=True):
    points = [
        (x_offset, y_offset),
        (x_offset + length, y_offset),
        (x_offset + length, y_offset + height),
        (x_offset, y_offset + height),
    ]
    flow_indicator_x_offset = x_offset + length / 2
    flow_indicator_y_offset = y_offset + height * 2.5 if layer == 'MUA-DUCT' else y_offset + height * 4
    flow_indicator_direction = 'down' if layer == 'MUA-DUCT' else 'up'
    if flow_indicator:
        draw_flow_indicator(msp, flow_indicator_x_offset, flow_indicator_y_offset, width=1, height=2, tail_length=6, layer='FLOW-INDICATOR', direction=flow_indicator_direction)
        if(layer == 'MUA-DUCT'):
            msp.add_mtext(f"{layer} \n {length}\"X{width}\"", dxfattribs={'insert': (x_offset + length / 2 - 10, y_offset + height * 10.5), 'color': colors.YELLOW, 'char_height': 1.5})
        else:
            msp.add_mtext(f"{layer} \n {length}\"X{width}\"", dxfattribs={'insert': (x_offset + length / 2 - 10, y_offset + height * 7.5), 'color': colors.YELLOW, 'char_height': 1.5})
    msp.add_lwpolyline(points, close=True, dxfattribs={'layer': layer})


def draw_flow_indicator(msp, base_x=0, base_y=0, width=1, height=2, tail_length=6, layer='FLOW-INDICATOR', direction='down'):
    if direction == 'up':
        triangle_points = [
            (base_x, base_y), 
            (base_x + width, base_y), 
            (base_x + width / 2, base_y + height), 
            (base_x, base_y), 
        ]
        tail_points = [
            (base_x + width / 2, base_y), 
            (base_x + width / 2, base_y - 2), 
            (base_x - width / 4, base_y - 2.5), 
            (base_x + width, base_y - 2.8), 
            (base_x + width / 2, base_y - 3), 
            (base_x + width / 2, base_y - tail_length), 
        ]
    elif direction == 'down':
        triangle_points = [
            (base_x, base_y),
            (base_x + width, base_y),
            (base_x + width / 2, base_y - height),
            (base_x, base_y),
        ]
        tail_points = [
            (base_x + width / 2, base_y),  
            (base_x + width / 2, base_y + 2),  
            (base_x - width / 4, base_y + 2.5),  
            (base_x + width, base_y + 2.8),  
            (base_x + width / 2, base_y + 3),   
            (base_x + width / 2, base_y + tail_length), 
        ]

    msp.add_lwpolyline(triangle_points, close=True, dxfattribs={'layer': layer})
    
    msp.add_lwpolyline(tail_points, close=False, dxfattribs={'layer': layer})
    
    hatch = msp.add_hatch(color=colors.RED)
    hatch.paths.add_polyline_path(triangle_points, is_closed=True)

def draw_hood_accessories(msp, x_offset=0, y_offset=0, layer='HOOD-ACCESSORIES'):
    box_points = [
        (x_offset, y_offset),
        (x_offset, y_offset + 2),
        (x_offset + 6, y_offset + 2),
        (x_offset + 6, y_offset),
    ]
    line_points = [
        (x_offset, y_offset + 1.75),
        (x_offset + 6, y_offset + 1.75)
    ]
    
    tool_points = [
        (x_offset + 2.25, y_offset + 1.75),
        (x_offset + 2.25, y_offset + 1.5),
        
        (x_offset + 2.25, y_offset + 1.75),
        
        (x_offset + 2.70, y_offset + 1.75),
        (x_offset + 2.70, y_offset + 1.5),
        
        (x_offset + 2.70, y_offset + 1.75),
        
        (x_offset + 3.55, y_offset + 1.75),
        (x_offset + 3.55, y_offset + 1.5),
        
        (x_offset + 3.55, y_offset + 1.75),
        
        (x_offset + 3.98, y_offset + 1.75),
        (x_offset + 3.98, y_offset + 1.5),
    ]
    
    msp.add_lwpolyline(box_points, close=True, dxfattribs={'layer': layer})
    msp.add_lwpolyline(line_points, close=True, dxfattribs={'layer': layer})
    msp.add_lwpolyline(tool_points, close=False, dxfattribs={'layer': layer})
    
    
    arc_points = [
        ((x_offset + 2.25, y_offset + 1.5, math.tan(math.radians(25))), (x_offset + 2.70, y_offset + 1.5, 0)),
        ((x_offset + 2.70, y_offset + 1.5, math.tan(math.radians(14))), (x_offset + 3.55, y_offset + 1.5, 0)),
        ((x_offset + 3.55, y_offset + 1.5, math.tan(math.radians(26))), (x_offset + 3.98, y_offset + 1.5, 0))
    ]

    for start, end in arc_points:
        points = [start, end]
        msp.add_lwpolyline(points, format='xyb', dxfattribs={'layer': layer})
        
    
    arc_line_points = [
        (x_offset + 2.25, y_offset + 1.395),
        (x_offset + 2.70, y_offset + 1.395),
        (x_offset + 3.55, y_offset + 1.395),
        (x_offset + 3.98, y_offset + 1.395),
    ]
    
    for index, (start, end) in enumerate(zip(arc_line_points, arc_line_points[1:])):
        if index == 0:
            start = ((start[0] + end[0]) / 2, start[1]) 
        elif index == 2:
            end = ((start[0] + end[0]) / 2, end[1]) 

        points = [start, end]
        msp.add_lwpolyline(points, close=False, dxfattribs={'layer': layer})
    
    arc_bottom_tool_points = [
        (x_offset + 2.8, y_offset + 1.395),
        (x_offset + 2.8, y_offset + 0.9),
        (x_offset + 2.88, y_offset + 0.82),
        (x_offset + 3.3, y_offset + 0.82),
        
        (x_offset + 3.4, y_offset + 0.9),
        (x_offset + 2.8, y_offset + 0.9),
        
        (x_offset + 3.4, y_offset + 0.9),
        (x_offset + 3.4, y_offset + 1.395),
    ]
    
    for index, (start, end) in enumerate(zip(arc_bottom_tool_points, arc_bottom_tool_points[1:])):
        points = [start, end]
        msp.add_lwpolyline(points, close=False, dxfattribs={'layer': layer})
    
    
    accessory_tower_points = [
        (x_offset + 2.8, y_offset + 1.75),
        (x_offset + 2.8, y_offset + 5.5),
        
        (x_offset + 2.8, y_offset + 1.75),
        
        (x_offset + 3.4, y_offset + 1.75),
        (x_offset + 3.4, y_offset + 5.5),
    ]
    msp.add_lwpolyline(accessory_tower_points, close=False, dxfattribs={'layer': layer})
    
    draw_accessories_tower_cap(msp, x_offset=x_offset + 2.8, y_offset=y_offset, tower_width=0.6, tower_height=5.5, layer=layer)
    

def draw_hood_top(msp, x_offset=0, y_offset=0, layer='HOOD', hood_length=110, hood_height=30, collars={}, cabinet={}, drain_location='', is_first_hood = True):
    cabinet_width = cabinet['CABINETWIDTH']
    
    if(drain_location == 'left'):
        points = [
            (x_offset - cabinet_width, y_offset),
            (x_offset + hood_length - (0 if is_first_hood else cabinet_width), y_offset),
            (x_offset + hood_length - (0 if is_first_hood else cabinet_width), y_offset - hood_height * 0.3),
            (x_offset - cabinet_width, y_offset - hood_height * 0.3),
        ]
    else:
        points = [
            (x_offset, y_offset),
            (x_offset + hood_length + (0 if drain_location == 'right' and is_first_hood else cabinet_width), y_offset),
            (x_offset + hood_length + (0 if drain_location == 'right' and is_first_hood else cabinet_width), y_offset - hood_height * 0.3),
            (x_offset, y_offset - hood_height * 0.3),
        ]
    
    msp.add_lwpolyline(points, close=True, dxfattribs={'layer': layer})
    
    exh_collar = collars['exh_collar']
    exh_collar_qty = collars['exh_collar_qty']
    # check if mua_collar is not empty
    mua_collar = collars['mua_collar']
    mua_collar_qty = collars['mua_collar_qty']
    
    collar_configs = []
    
    for i in range(mua_collar_qty):
        if mua_collar_qty == 1:
            config = {
                'x_offset': x_offset + (hood_length * 0.1) + ((hood_length * 0.24) + (hood_length * 0.04) + (hood_length * 0.24) + (hood_length * 0.04)) * i, 
                'layer': 'MUA-DUCT',
                'length': mua_collar['MUACOLLARLENGTH'],
                'width': mua_collar['MUACOLLARWIDTH']
            }
            collar_configs.append(config)
            if(i == 1):
                break
        elif mua_collar_qty >= 2:
            custom_x_offset = x_offset + (hood_length * 0.1) + ((hood_length * 0.04) + (hood_length * 0.24)) * i
            if(i == 1):
                i = 2
                custom_x_offset = x_offset + (hood_length * 0.1) + ((hood_length * 0.04) + (hood_length * 0.24)) * i
            elif(i == 2):
                i = 1
                custom_x_offset = x_offset + (hood_length * 0.1) + ((hood_length * 0.04) + (hood_length * 0.24)) * i
            # elif(custom_x_offset >= hood_length):
            #     custom_x_offset = x_offset + (hood_length * 0.1) + ((hood_length * 0.04) + (hood_length * 0.24)) * 2
            config = {
                'x_offset': custom_x_offset, 
                'layer': 'MUA-DUCT',
                'length': mua_collar['MUACOLLARLENGTH'],
                'width': mua_collar['MUACOLLARWIDTH']
            }
            collar_configs.append(config)
    
    for i in range(exh_collar_qty):
        if exh_collar_qty == 1:
            config = {
                'x_offset': x_offset + (hood_length * 0.1) + (hood_length * 0.24) + (hood_length * 0.04), 
                'layer': 'EXHAUST-DUCT',
                'length': exh_collar['EXHCOLLARLENGTH'],
                'width': exh_collar['EXHCOLLARWIDTH']
            }
            collar_configs.append(config)
            break
        config = {
            'x_offset': x_offset + (hood_length * 0.15) + ((hood_length * 0.24) + (hood_length * 0.05)) * i, 
            'layer': 'EXHAUST-DUCT',
            'length': exh_collar['EXHCOLLARLENGTH'],
            'width': exh_collar['EXHCOLLARWIDTH']
        }
        collar_configs.append(config)
    
    for config in collar_configs:
        draw_hood_top_collar(msp, x_offset=config['x_offset'], y_offset=y_offset, length=config['length'], height=3, width=config['width'], layer=config['layer'])
    
    draw_hood_accessories(msp, x_offset=x_offset + (hood_length * 0.005), y_offset=y_offset, layer='HOOD-ACCESSORIES')
    draw_hood_accessories(msp, x_offset=x_offset + (hood_length * 0.935), y_offset=y_offset, layer='HOOD-ACCESSORIES')
    


def draw_hood_label(msp, hood_length=110, hood_height=30, x_offset=0, y_offset=0, layer='HOOD-LABEL', label='CONT. CAPTURE', rotated_angle=90):
    msp.add_text(
        label, 
        dxfattribs={
            'layer': layer,
            'insert': (x_offset - 1, y_offset + 1),
            'height': 1,
            'rotation': rotated_angle
        }
    )
    
    msp.add_line(
        start=(x_offset - 0.5, y_offset),
        end=(x_offset - 0.5, y_offset + 13),
        dxfattribs={'layer': layer, 'color': colors.RED}
    )


def draw_hood_body(msp, x_offset=0, y_offset=0, layer='HOOD', hood_length=110, hood_height=30, cabinet={}, hood_info={}, is_last_hood=False, is_first_hood=True):
    
    hood_no = hood_info['HOOD_NO']
    
    cabinet_type = cabinet['CONTROLTYPE']
    
    points = [
        (x_offset, y_offset - hood_height * .3),
        (x_offset, y_offset - hood_height),
        (x_offset + hood_length, y_offset - hood_height),
        (x_offset + hood_length, y_offset - hood_height * .3),
    ]
    
    indicators = int(hood_length / 11)
    
    for i in range(indicators):
        draw_flow_indicator(msp, (x_offset + 6) + (i * 11) , y_offset - (hood_height * 0.5) - 1, width=1, height=2, tail_length=6, layer='FLOW-INDICATOR')
    
    draw_hood_label(
        msp, 
        hood_length=hood_length, hood_height=hood_height, 
        x_offset=x_offset + hood_length, y_offset=y_offset - hood_height, 
        layer='HOOD-LABEL', label=cabinet_type if is_last_hood else "CONT. CAPTURE",
        rotated_angle=90
    )

    msp.add_text(
        f"HOOD #{hood_no}", 
        dxfattribs={
            'layer': 'HOOD-ACCESSORIES',
            'insert': (x_offset + hood_length * 0.15, y_offset - hood_height * 0.85),
        })
    msp.add_lwpolyline(points, close=False, dxfattribs={'layer': layer})


def draw_hood(msp, x_offset=0, y_offset=0, layer='HOOD', hood_length=110, hood_height=30, collars={}, cabinet={}, drain_location = '', hood_info={}, is_last_hood=False, is_first_hood=True):
    cabinet_width = cabinet['CABINETWIDTH']
    custom_x_offset = x_offset
    if(drain_location == 'left'):
        if(is_first_hood):
            custom_x_offset = x_offset
        else:   
            custom_x_offset = x_offset + cabinet_width
    draw_hood_top(msp, custom_x_offset, y_offset, layer, hood_length, hood_height, collars=collars, cabinet=cabinet, drain_location=drain_location, is_first_hood=is_first_hood)
    draw_hood_body(msp, x_offset, y_offset, layer, hood_length, hood_height, cabinet=cabinet, hood_info=hood_info, is_last_hood=is_last_hood, is_first_hood=is_first_hood)
    

def draw_tower(block, tower_height, tower_width, number_of_floors):
    floor_height = tower_height / number_of_floors

    for i in range(number_of_floors):
        y = i * floor_height
        block.add_lwpolyline([(0, y), (tower_width, y + floor_height)], dxfattribs={'layer': 'DIM'})
        
        block.add_lwpolyline([(0, y), (0, y + floor_height)], dxfattribs={'layer': 'DIM'})
        block.add_lwpolyline([(tower_width, y), (tower_width, y + floor_height)], dxfattribs={'layer': 'DIM'})


def draw_tower_cap(block, tower_height, tower_width):
    cap_points = [
        (-0.3 * tower_width, tower_height),                       
        (tower_width * 0.3, tower_height),                        
        (tower_width * 0.45, tower_height - 0.3 * tower_width),   
        (tower_width * 0.65, tower_height + 0.02 * tower_height),  
        (tower_width * 0.7, tower_height),                        
        (tower_width * 1.3, tower_height)                         
    ]
    block.add_lwpolyline(cap_points, dxfattribs={'layer': 'DIM'})
    

def draw_rectangle(msp, points, layer):
    msp.add_lwpolyline(points, close=True, dxfattribs={'layer': layer})


def get_rectangle_points(x, y, width, height):
    return [
        (x, y),
        (x + width, y),
        (x + width, y + height),
        (x, y + height),
    ]


def draw_switch_panel(msp, x, y, width, height, layer):
    panel_points = get_rectangle_points(x, y, width, height)
    draw_rectangle(msp, panel_points, layer)


def draw_switch(msp, x, y, width, height, layer):
    switch_points = get_rectangle_points(x, y, width, height)
    switch_top_points = [
        (x + width, y + height / 1.5),
        (x, y + height / 1.5),
    ]
    draw_rectangle(msp, switch_points, layer)
    draw_rectangle(msp, switch_top_points, layer)

def draw_front_elevation_info_label(msp, text='', start_x_offset=0, start_y_offset=0, pointer_x_offset=0, pointer_y_offset=0, alignment='left'):
    ml_builder = msp.add_multileader_mtext("Standard")
    ml_builder.set_content(
        text,
        alignment= mleader.TextAlignment.right if alignment == 'right' else mleader.TextAlignment.left,
        color=colors.YELLOW,
        char_height=2.0,
    )
    ml_builder.add_leader_line(mleader.ConnectionSide.right if alignment == 'right' else mleader.ConnectionSide.left, [Vec2(pointer_x_offset, pointer_y_offset)])
    ml_builder.set_arrow_properties(size=2.0)
    ml_builder.set_leader_properties(
        color=colors.CYAN,
        # linetype="DASHEDX2",
        lineweight=5,
    )
    
    ml_builder.build(insert=Vec2(start_x_offset, start_y_offset))

def draw_front_elevation_hood_cabinet(msp, x_offset=0, y_offset=0, layer='HOOD', width=12, height=22, drain_location='right'):
    points = get_rectangle_points(x_offset, y_offset, width, height)
    draw_rectangle(msp, points, layer)

    switch_panel_width = 5
    switch_panel_height = 5

    switch_panel_x = x_offset + width / 3
    switch_panel_y = y_offset + height / 3 - switch_panel_height
    draw_switch_panel(msp, switch_panel_x, switch_panel_y, switch_panel_width, switch_panel_height, 'HOOD-ACCESSORIES')

    switch_width = 0.5
    switch_height = 1.5

    switch_x1 = switch_panel_x + switch_panel_width / 3.5
    switch_y1 = switch_panel_y + switch_panel_height / 2 - switch_height
    draw_switch(msp, switch_x1, switch_y1, switch_width, switch_height, 'HOOD-ACCESSORIES')

    switch_x2 = switch_panel_x + switch_panel_width / 1.5
    switch_y2 = switch_panel_y + switch_panel_height / 2 - switch_height
    draw_switch(msp, switch_x2, switch_y2, switch_width, switch_height, 'HOOD-ACCESSORIES')
    
    if(drain_location == 'left'):
        draw_front_elevation_info_label(
            msp, 
            text=f"SWITCH PANEL\n1 LIGHT, 1 FAN", 
            start_x_offset=switch_x1 - 20, start_y_offset=switch_y1 - 6, 
            pointer_x_offset=switch_x1 - 2, pointer_y_offset=switch_y1,
            alignment='right'
        )
    else:
        draw_front_elevation_info_label(
            msp, 
            text=f"SWITCH PANEL\n1 LIGHT, 1 FAN", 
            start_x_offset=switch_x1 + 20, start_y_offset=switch_y1 - 6, 
            pointer_x_offset=switch_x1 + 5, pointer_y_offset=switch_y1,
            alignment='left'
        )
    
    # msp.add_mtext(
    #     f"SWITCH PANEL\n1 LIGHT, 1 FAN", 
    #     dxfattribs={
    #         'layer': 'HOOD-ACCESSORIES',
    #         'insert': (switch_x1 + 20, switch_y1 - 8),
    #     }
    # )
    
    # draw_left_arrow(
    #     msp, 
    #     base_x=switch_x1 + 20 - 3, base_y=switch_y1 - 8, 
    #     end_x=switch_panel_x + switch_panel_width + 1, end_y=switch_panel_y,
    #     width=switch_width, height=switch_height, 
    #     direction='left'
    # )
    
    draw_vertical_dimension_line(
        msp, 
        x_offset, y_offset + height, 
        height=height, width=width, 
        val_top=f"{width}", val_center=inches_to_feet_and_inches(f"{width}"), 
        layer='HOOD-ACCESSORIES'
    )
    

def draw_left_arrow(msp, base_x=0, base_y=0, end_x=0, end_y=0, width=1, height=2, layer='INDICATOR', direction='left'):
    if direction == 'left':
        triangle_points = [
            (end_x, end_y), 
            (end_x, end_y + height), 
            (end_x - width, end_y + height / 2), 
            (end_x, end_y), 
        ]
        
        tail_points = [
            (base_x, base_y + height / 2),
            (end_x - width, end_y + height / 2),
        ]

    msp.add_lwpolyline(triangle_points, close=True, dxfattribs={'layer': layer})
    
    msp.add_lwpolyline(tail_points, close=False, dxfattribs={'layer': layer})
    
    hatch = msp.add_hatch(color=colors.CYAN)
    hatch.paths.add_polyline_path(triangle_points, is_closed=True)

def draw_front_elevation(form_data, doc = ezdxf.new(dxfversion='R2010'), x_offset=0, y_offset=0):
    
    global hood_total_length
    # doc = ezdxf.new(dxfversion='R2010')
    msp = doc.modelspace()
    
    print(f"x_offset: {x_offset}, y_offset: {y_offset}")
    
    block = doc.blocks.new(name='TOWER_BLOCK')
    tower_height = form_data['tower_height']
    tower_width = form_data['tower_width']
    number_of_hoods = form_data['number_of_hoods']
    hood_information = form_data['HOOD_INFORMATION']
    cabinet_information = form_data['CABINET_INFORMATION']
    hood_schedule_information = form_data['HOOD_SCHEDULE_INFO']
    number_of_floors = 15

    draw_tower(block, tower_height, tower_width, number_of_floors)
    draw_tower_cap(block, tower_height, tower_width)
    
    hood_total_length = 0
    
    for i in range(number_of_hoods):
        hood_height = hood_information[i]['HOODHEIGHT']
        hood_length = hood_information[i]['HOODLENGTH']
        drain_location = hood_information[0]['DRAIN']
        exh_collar = hood_information[i]['EXH_COLLAR']
        exh_collar_qty = hood_information[i]['EXH_COLLAR_QTY']
        mua_collar = hood_information[i]['MUA_COLLAR']
        mua_collar_qty = hood_information[i]['MUA_COLLAR_QTY']
        
        collars = {
            'exh_collar': exh_collar,
            'exh_collar_qty': exh_collar_qty,
            'mua_collar': mua_collar,
            'mua_collar_qty': mua_collar_qty
        }
        
        current_x_offset = tower_width + hood_total_length
        hood_total_length += hood_length
        
        current_y_offset = y_offset + tower_height * 0.8
        cabinet = cabinet_information[i]
        cabinet_width = cabinet["CABINETWIDTH"]
        cabinet_height = cabinet["CABINETHEIGHT"]
        
        if i == 0:
            draw_horizontal_dimension_line_left(msp, current_x_offset, current_y_offset, hood_height, dist=45, val=f"{hood_height} {inches_to_feet_and_inches(f'{hood_height}')}", layer='HOOD-ACCESSORIES')
            draw_horizontal_dimension_line_left(msp, current_x_offset, current_y_offset - hood_height, (tower_height * 0.8) - hood_height, dist=45, val=f"{(tower_height * 0.8) - hood_height} {inches_to_feet_and_inches(f'{int((tower_height * 0.8) - hood_height)}')}", layer='HOOD-ACCESSORIES')
        if(drain_location == 'left'):
            current_x_offset = current_x_offset + cabinet_width
        
        is_first_hood = False
        
        if(i == 0):
            is_first_hood = True
            if(number_of_hoods == 1):
                is_first_hood = False
            if(drain_location == 'left'):
                if(number_of_hoods == 1):
                    is_first_hood = True

        draw_hood(
            msp, 
            current_x_offset, current_y_offset, 
            hood_length=hood_length, hood_height=hood_height, 
            collars=collars, cabinet=cabinet, 
            drain_location=drain_location,
            hood_info=hood_schedule_information[i],
            is_last_hood=(number_of_hoods - i == 1),
            is_first_hood=is_first_hood
        )
        
        draw_vertical_dimension_line(
            msp, 
            current_x_offset, current_y_offset, 
            height=hood_height, width=hood_length, 
            val_top=f"{hood_length}", val_center=inches_to_feet_and_inches(f"{hood_length}"), 
            layer='HOOD-ACCESSORIES'
        )
        
        msp.add_lwpolyline(
            [(0, y_offset), ( hood_total_length + tower_width + cabinet_width * 2, y_offset)], 
            close=False, dxfattribs={'layer': 'DIM'}
        )
    
    
    if drain_location == 'left':
        draw_front_elevation_hood_cabinet(msp, x_offset=tower_width, y_offset=current_y_offset - hood_height, width=cabinet_width, height=hood_height - hood_height * 0.3, drain_location=drain_location) 
    else:
        draw_front_elevation_hood_cabinet(msp, x_offset=current_x_offset + hood_length, y_offset=current_y_offset - hood_height, width=cabinet_width, height=hood_height - hood_height * 0.3, drain_location=drain_location) 

    draw_horizontal_dimension_line_right(msp, current_x_offset + hood_length, current_y_offset, hood_height, val=f"{hood_height} {inches_to_feet_and_inches(f'{hood_height}')}", layer='HOOD-ACCESSORIES')
    msp.add_blockref('TOWER_BLOCK', (x_offset, y_offset), dxfattribs={'layer': 'DIM', 'color': colors.LIGHT_GRAY})
    add_section_header(msp, text='FRONT ELEVATION', x_offset=x_offset + 50, y_offset=y_offset - 25, font_height=6)
    
    return hood_total_length

# FRONT ELEVATION CODE END

